#!/usr/bin/env python
import string
import sys
import copy
from gzip import open as _gzip_open
#########################################################################################################################
# Extend the XA field of a bwa output sam file. All concordant pairs are removed. Singleton reads and discordant
# reads are kept and extended.
########################################################################################################################
def maybe_gzip_open(filename, *args, **kwargs):
       if filename.endswith('.gz'):
               return _gzip_open(filename, *args, **kwargs)
       elif filename == '-':
               return sys.stdin
       else:
               return open(filename, *args, **kwargs)

class sam:
	def __init__(self):
		self.sam_line=""
		self.name=""
		self.flag=0
		self.strand=0
		self.unmap=False
		self.is_dup=False
		self.chr=""
		self.pos=0
		self.mapq=""
		self.cigar=""
		self.mchr=""
		self.mpos=0
		self.isize=0
		self.seq=""
		self.qval=""
		self.XA=""
		self.difficult=0
		self.mismatch=-1
		#self.difficult=False
		self.is_bad=False
		self.x0_num=0
		self.xa_field=""
BAD_MM_NUM=8
BAD_X0_NUM=3
SINGLE_INDEL = 1
MULTI_INDEL = 2
SOFT_CLIPING = 3
EXCESSIVE_MISMATCH = 4
UN_MAP = 5
def usage():
	print "%s [bwa sam file] [output folder] [mean of insert size] [std of insert size]" % sys.argv[0]
def is_concordant(strand1,strand2,isize1,con_thresh):
	if (isize1<= con_thresh and isize1>0 and strand1==0 and strand2==1)or (isize1>=-1*con_thresh and isize1<0 and strand1==1 and strand2==0):
		#print isize1,con_thresh,isize1<con_thresh
		return True
	else:
		return False
def is_difficult(cigar):
	#if (cigar.find("I")>=0 or cigar.find("D")>=0 or cigar.find("S")>=0):
	#	return True
	#else:
	#	return False
	n_S = cigar.count("S")
	n_ID = cigar.count("I") + cigar.count("D")
	if n_S > 0:
		tmp=""
		len_S=0
		for i in range(0,len(cigar)):
			if cigar[i].isdigit():
				tmp = tmp + cigar[i]
			else:
				if cigar[i] == "S" and string.atoi(tmp)>len_S:
					len_S = string.atoi(tmp)
					
				else:
					tmp = ""
		if len_S > 0:
			return SOFT_CLIPING
	if n_ID == 1 :
		return SINGLE_INDEL
	if n_ID > 1 :
		return MULTI_INDEL 
	if cigar == "*":
		return UN_MAP
	return 0 
def is_large_del(strand1,strand2,isize1,con_thresh):
	if (isize1 > con_thresh and strand1==0 and strand2==1) or (isize1 < -1*con_thresh and strand1==1 and strand2==0):
		return True
	else:
		return False
def is_dup (strand1,strand2,isize1):
	if(isize1 < 0 and strand1==0 and strand2==1) or (isize1 > 0 and strand1==1 and strand2==0):
		return True
	else:
		return False
def is_inv(strand1,strand2):
	if (strand1==strand2):
		return True
	else:
		return False
def load_read(line, mate_list):
	tmp1=line.split()
        a=sam()
        a.sam_line=line
        #print "sam line1:%s" % line1
        a.name=tmp1[0]
        a.flag=string.atoi(tmp1[1])
        a.strand=(a.flag & 0x10)/0x10
        a.unmap=(a.flag & 0x04)/0x04
	a.is_dup=(a.flag & 0x0400)/0x0400
	if a.is_dup:
		return
        a.chr=tmp1[2]
        a.pos=string.atoi(tmp1[3])
        a.mapq=tmp1[4]
        a.cigar=tmp1[5]
        a.mchr=tmp1[6]
        a.mpos=string.atoi(tmp1[7])
        a.isize=string.atoi(tmp1[8])
        a.seq=tmp1[9]
        a.qval=tmp1[10]
        a.difficult=is_difficult(a.cigar)
	for i in range(11,len(tmp1)):
        	#if (tmp1[i].find('NM:'))>=0:
                #	if string.atoi(tmp1[i][5:])>BAD_MM_NUM:
                #       	a.difficult = True
        	if (tmp1[i].find('MD:'))>=0:
			mismatch = (tmp1[i][5:]).upper().count("A") + (tmp1[i][5:]).upper().count("C") + (tmp1[i][5:]).upper().count("G") + (tmp1[i][5:]).upper().count("T")
			a.mismatch = mismatch
                	if a.difficult == 0 and mismatch>BAD_MM_NUM:
                        	a.difficult = EXCESSIVE_MISMATCH
                if (tmp1[i].find('X0:'))>=0:
			a.x0_num=string.atoi(tmp1[i][5:])
                        if a.x0_num>BAD_X0_NUM:
                                a.is_bad = True
                if (tmp1[i].find('XA'))>=0:
			a.xa_field=tmp1[i]
	mate_list.append(a)
	if a.x0_num>1 and len(a.xa_field)==0:
		return
	#mate_list.append(a)
	tmp_xa1=a.xa_field[5:].split(";")
		
	for j in range(0,a.x0_num-1):
                if len(tmp_xa1[j])>0:
                        tmp_xa2=tmp_xa1[j].split(",")
                        c=sam()
			c.name=a.name
                        if tmp_xa2[1][0:1]=="+":
                                c.strand=0
                        else :
                                c.strand=1
                        c.chr=tmp_xa2[0]
			if c.chr != a.chr:
				continue
                        c.pos=string.atoi(tmp_xa2[1][1:])
			c.mapq=a.mapq
                        c.cigar=tmp_xa2[2]
			c.seq=a.seq
			c.qval=a.qval
        		c.difficult=is_difficult(c.cigar)
			if c.difficult>0:
				mate_list[0].difficult=c.difficult
			mate_list.append(c)

def parse_XA(sam):
        tmp_xa1=sam.xa_field.split(";")

        #for j in range(0,len(tmp_xa1)):
        for j in range(0,sam.x0_num):
        	if len(tmp_xa1[j])>0:
                	tmp_xa2=tmp_xa1[j].split(",")
                        #print "xa1:%s" % tmp_xa1[j]
                        #print "xa2:%s" % tmp_xa2[0]
                        #print "xa2:%s" % tmp_xa2[1]
                        #print "xa2:%s" % tmp_xa2[2]
                        c=sam()
                        if tmp_xa2[1][0:1]=="+":
                        	c.strand=0
                        else :
                        	c.strand=1
                        c.chr=tmp_xa2[0]
                        c.pos=string.atoi(tmp_xa2[1][1:])
                        c.cigar=tmp_xa2[2]
	return c
def rc_seq(seq0):
	seq=seq0.upper()
        tmpseq=seq.replace("A","a")
	seq=tmpseq
        tmpseq=seq.replace("T","A")
	seq=tmpseq
        tmpseq=seq.replace("a","T")
	seq=tmpseq
	tmpseq=seq.replace("G","g")
	seq=tmpseq
        tmpseq=seq.replace("C","G")
	seq=tmpseq
        tmpseq=seq.replace("g","C")
	seq=tmpseq[::-1]
	return seq
def extend_XA(filename0,out_folder,con_thresh):
	f1=maybe_gzip_open(filename0,'r')
	#f1=open(filename0,'r')
	#fout=open(filename+"_extend_difficult","w")
	tmp=filename0.split("/")
	filename=tmp[-1]
	fout_single=open(out_folder+"/"+filename+"_single","w")
	fout_del=open(out_folder+"/"+filename+"_large","w")
	fout_inv=open(out_folder+"/"+filename+"_inver","w")
	fout_dup=open(out_folder+"/"+filename+"_dup","w")
	mate1_list=[]
	mate2_list=[]	
	single_read_line =0
	pair_num = 0
	while True:
    		line1 = f1.readline()
		if line1[0:1]=='@':
			continue
    		if len(line1) == 0: # Zero length indicates EOF
        		break
    		#line2 = f1.readline()
		#if line2[0:1]=='@':
		#	print "line 2 contains @"
		#	sys.exit(1)	
    		#if len(line2) == 0: # Zero length indicates EOF
			#print "odd line num"
			#sys.exit(1)	
		#	break
		name1 = line1.split()[0]
		while True:
    			line2 = f1.readline()
			if line2[0:1]=='@':
				print "line 2 contains @"
				sys.exit(1)	
    			if len(line2) == 0: # Zero length indicates EOF
				#print "odd line num"
				#sys.exit(1)	
				break
			name2 = line2.split()[0]
			if name1 == name2:
				pair_num+=1
				break
			else:
				single_read_line += 1
				line1 = line2
				name1 = name2
		if len(line2) == 0: # Zero length indicates EOF
			break
						
		load_read(line1,mate1_list)
		load_read(line2,mate2_list)
		
		if len(mate1_list)==0 or len(mate2_list)==0:
			mate1_list=[]
                        mate2_list=[]
                        continue
		a=mate1_list[0]
                b=mate2_list[0]

		if a.name != b.name:
			print "file not in pair."
			sys.exit(1);
		#is_concordant(pos1,pos2,strand1,strand2,isize1,con_thresh)
		#print a.difficult
		#print b.difficult
		#if (a.unmap or a.difficult>0) and  ( b.unmap or b.difficult>0):
		if a.unmap  and   b.unmap:
			mate1_list=[]
                        mate2_list=[]
                        continue
		#print x0_num_a
		#print x0_num_b

		
		buffer1=[]
		buffer2=[]
		type=[]
		is_con=False
		is_single=False
		discordant_type=0#1=del 2=dup 3=inv
		if a.difficult > 0 or b.difficult > 0 or a.unmap or b.unmap:
			is_single=True

		loop_a_len = len(mate1_list)
		loop_b_len = len(mate2_list)
		if a.difficult:
			loop_b_len = 1
		if b.difficult:
			loop_a_len = 1
		#for i in range(0,len(mate1_list)):		
		for i in range(0,loop_a_len):		
			#for j in range (0,len(mate2_list)):
			for j in range (0,loop_b_len):
				#print "flag1,flag2:%s,%s" % (mate1_list[i].flag,mate2_list[i].flag)
				#d=mate1_list[i]
                                #e=mate2_list[j]
				
				if a.difficult > 0 or a.unmap:
				#if a.difficult == SOFT_CLIPING or a.difficult == UN_MAP:
                                        d=mate1_list[i]
                                        e=mate2_list[j]
					
                                        fout_single.write(e.name+"\t")
                                        #fout_single.write(str(e.flag)+"\t")
					#fout_single.write(str(e.strand*0x20+e.strand*0x10+(b.flag & 0x7CF))+"\t")
					fout_single.write(str(e.strand*0x10)+"\t")
                                        fout_single.write(e.chr+"\t")
                                        fout_single.write(str(e.pos)+"\t")
                                        fout_single.write(e.mapq+"\t")
                                        fout_single.write(e.cigar+"\t")
                                        fout_single.write("="+"\t")
                                        fout_single.write(str(e.pos)+"\t")
                                        fout_single.write("0\t")
					fout_single.write(e.seq+"\t")
                                        fout_single.write(e.qval+"\t")
                                        fout_single.write("DF:i:"+str(d.difficult)+"\t")
                                        fout_single.write("MM:i:"+str(a.mismatch)+"\t")
					fout_single.write("R2:Z:")
					if d.strand==0:
                                                fout_single.write(d.seq+"\t")
                                        else:
                                                fout_single.write(rc_seq(d.seq)+"\t")
					fout_single.write("\n")

				#elif b.difficult == SOFT_CLIPING or b.difficult == UN_MAP:
				elif b.difficult > 0 or b.unmap:
					e=mate1_list[i]
                                        d=mate2_list[j]
					fout_single.write(e.name+"\t")
					#fout_single.write(str(e.strand*0x20+e.strand*0x10+(a.flag & 0x7CF))+"\t")
					fout_single.write(str(e.strand*0x10)+"\t")
                                        #fout_single.write(str(e.flag)+"\t")
                                        fout_single.write(e.chr+"\t")
                                        fout_single.write(str(e.pos)+"\t")
                                        fout_single.write(e.mapq+"\t")
                                        fout_single.write(e.cigar+"\t")
                                        fout_single.write("="+"\t")
                                        fout_single.write(str(e.pos)+"\t")
                                        fout_single.write("0\t")
                                        fout_single.write(e.seq+"\t")
                                        fout_single.write(e.qval+"\t")
                                        fout_single.write("DF:i:"+str(d.difficult)+"\t")
                                        fout_single.write("MM:i:"+str(b.mismatch)+"\t")
                                        fout_single.write("R2:Z:")
                                        if d.strand==0:
                                                fout_single.write(d.seq+"\t")
                                        else:
                                                fout_single.write(rc_seq(d.seq)+"\t")
                                        fout_single.write("\n")



				else:
					if mate1_list[i].chr == mate2_list[j].chr:
						d=mate1_list[i]
						e=mate2_list[j]
						#if abs(d.pos - e.pos)<con_thresh:
						if d.strand==0:
							pos5_1=d.pos
						else :
							pos5_1=d.pos + len(d.seq)
						if e.strand==0:
							pos5_2=e.pos
						else :
							pos5_2=e.pos + len(e.seq)
						d.isize=pos5_2-pos5_1
						e.isize=pos5_1-pos5_2
						if is_concordant(d.strand,e.strand,d.isize,con_thresh):
							is_con=True
							break
						elif is_large_del(d.strand,e.strand,d.isize,con_thresh):
							discordant_type=1
							type.append(discordant_type)
                                		elif is_dup(d.strand,e.strand,d.isize):
							discordant_type=2
							type.append(discordant_type)
                                		elif is_inv(d.strand,e.strand):
							discordant_type=3
							type.append(discordant_type)
						elif d.isize==0:
							is_con=True
						else:
							print "error"
							print d.strand,e.strand,d.isize,con_thresh
							sys.exit(1)
						#print pos5_1,pos5_2
						tmpstr=""
						tmpstr=tmpstr+a.name+"\t"
						tmpstr=tmpstr+(str(e.strand*0x20+d.strand*0x10+(a.flag & 0x7CF))+"\t")
						tmpstr=tmpstr+(d.chr+"\t")
						tmpstr=tmpstr+(str(d.pos)+"\t")
						tmpstr=tmpstr+(a.mapq+"\t")
						tmpstr=tmpstr+(d.cigar+"\t")
						tmpstr=tmpstr+("="+"\t")
						tmpstr=tmpstr+(str(e.pos)+"\t")
						tmpstr=tmpstr+(str(d.isize)+"\t")
						tmpstr=tmpstr+(a.seq+"\t")
						tmpstr=tmpstr+(a.qval+"\n")
						buffer1.append(tmpstr)
						
						tmpstr=""
						tmpstr=tmpstr+(b.name+"\t")
        		                        tmpstr=tmpstr+(str(d.strand*0x20+e.strand*0x10+(b.flag & 0x7CF))+"\t")
                		                tmpstr=tmpstr+(e.chr+"\t")
                        		        tmpstr=tmpstr+(str(e.pos)+"\t")
	                        	        tmpstr=tmpstr+(b.mapq+"\t")
	        	                        tmpstr=tmpstr+(e.cigar+"\t")
        		       	                tmpstr=tmpstr+("="+"\t")
                        		        tmpstr=tmpstr+(str(d.pos)+"\t")
                                		tmpstr=tmpstr+(str(e.isize)+"\t")
	                                	tmpstr=tmpstr+(b.seq+"\t")
	        	                        tmpstr=tmpstr+(b.qval+"\n")				
						buffer2.append(tmpstr)
						
			
			if is_con:
				break
			
		if not is_single and not is_con:
			for i in range(0,len(buffer1)):
				if type[i]==1:
					fout_del.write(buffer1[i])
					fout_del.write(buffer2[i])
                                if type[i]==2:
					fout_dup.write(buffer1[i])
					fout_dup.write(buffer2[i])
                                if type[i]==3:
					fout_inv.write(buffer1[i])
					fout_inv.write(buffer2[i])

		#else:
			#print d.name, "is concordant:", 
		mate1_list=[]
		mate2_list=[]
	print "single read line num:%i\tpair read line num:%i\n" % (single_read_line,pair_num)
	if float(single_read_line)/float(pair_num) > 0.5:
		print "file seems not in pair"
	f1.close()
	fout_single.close()
	fout_del.close()
	fout_inv.close()
	fout_dup.close()
		#print "mate2"
		#for i in range(0,len(mate2_list)):		
		#	print mate2_list[i].pos


if __name__=='__main__':
	if len(sys.argv)!=5:
		usage()
		sys.exit(1)
	### bwa output sam file ###
	file1=sys.argv[1]
	### out put file ###
	out_folder=sys.argv[2]
	### the min length of mapping distance of a discordant pair, only for opp-in pairs ###
	mean=string.atoi(sys.argv[3])
	std=string.atoi(sys.argv[4])
	concordant_thresh=mean + 3*std
	extend_XA(file1,out_folder,concordant_thresh)
