#!/usr/bin/env python
import string
import sys
import numpy
from gzip import open as _gzip_open
#########################################################################################################################
# Extend the XA field of a bwa output sam file. All concordant pairs are removed. Singleton reads and discordant
# reads are kept and extended.
########################################################################################################################
def maybe_gzip_open(filename, *args, **kwargs):
       if filename.endswith('.gz'):
               return _gzip_open(filename, *args, **kwargs)
       elif filename == '-':
               return sys.stdin
       else:
               return open(filename, *args, **kwargs)

class sam:
	def __init__(self):
		self.sam_line=""
		self.name=""
		self.flag=0
		self.strand=0
		self.unmap=False
		self.chr=""
		self.pos=0
		self.mapq=""
		self.cigar=""
		self.mchr=""
		self.mpos=0
		self.isize=0
		self.seq=""
		self.qval=""
		self.XA=""
		self.difficult=0
		self.mismatch=-1
		#self.difficult=False
		self.is_bad=False
		self.x0_num=0
		self.xa_field=""
BAD_MM_NUM=8
BAD_X0_NUM=3
SINGLE_INDEL = 1
MULTI_INDEL = 2
SOFT_CLIPING = 3
EXCESSIVE_MISMATCH = 4
UN_MAP = 5
def usage():
	print "%s <bwa sam file> [lower bound of isize] [upper bound of isize] [read num used to calculate]" % sys.argv[0]
def is_concordant(strand1,strand2,isize1,con_thresh):
	if (isize1<= con_thresh and isize1>0 and strand1==0 and strand2==1)or (isize1>=-1*con_thresh and isize1<0 and strand1==1 and strand2==0):
		#print isize1,con_thresh,isize1<con_thresh
		return True
	else:
		return False
def is_difficult(cigar):
	#if (cigar.find("I")>=0 or cigar.find("D")>=0 or cigar.find("S")>=0):
	#	return True
	#else:
	#	return False
	n_S = cigar.count("S")
	n_ID = cigar.count("I") + cigar.count("D")
	if n_S > 0:
		tmp=""
		len_S=0
		for i in range(0,len(cigar)):
			if cigar[i].isdigit():
				tmp = tmp + cigar[i]
			else:
				if cigar[i] == "S" and string.atoi(tmp)>len_S:
					len_S = string.atoi(tmp)
					
				else:
					tmp = ""
		if len_S > 0:
			return SOFT_CLIPING
	if n_ID == 1 :
		return SINGLE_INDEL
	if n_ID > 1 :
		return MULTI_INDEL 
	if cigar == "*":
		return UN_MAP
	return 0 
def is_large_del(strand1,strand2,isize1,con_thresh):
	if (isize1 > con_thresh and strand1==0 and strand2==1) or (isize1 < -1*con_thresh and strand1==1 and strand2==0):
		return True
	else:
		return False
def is_dup (strand1,strand2,isize1):
	if(isize1 < 0 and strand1==0 and strand2==1) or (isize1 > 0 and strand1==1 and strand2==0):
		return True
	else:
		return False
def is_inv(strand1,strand2):
	if (strand1==strand2):
		return True
	else:
		return False
def load_read(line, a):
	tmp1=line.split()
        #a=sam()
        a.sam_line=line
        #print "sam line1:%s" % line1
        a.name=tmp1[0]
        a.flag=string.atoi(tmp1[1])
        a.strand=(a.flag & 0x10)/0x10
        a.unmap=(a.flag & 0x04)/0x04
        a.chr=tmp1[2]
        a.pos=string.atoi(tmp1[3])
        a.mapq=tmp1[4]
        a.cigar=tmp1[5]
        a.mchr=tmp1[6]
        a.mpos=string.atoi(tmp1[7])
        a.isize=string.atoi(tmp1[8])
        a.seq=tmp1[9]
        a.qval=tmp1[10]
        a.difficult=is_difficult(a.cigar)

def parse_XA(sam):
        tmp_xa1=sam.xa_field.split(";")

        #for j in range(0,len(tmp_xa1)):
        for j in range(0,sam.x0_num):
        	if len(tmp_xa1[j])>0:
                	tmp_xa2=tmp_xa1[j].split(",")
                        #print "xa1:%s" % tmp_xa1[j]
                        #print "xa2:%s" % tmp_xa2[0]
                        #print "xa2:%s" % tmp_xa2[1]
                        #print "xa2:%s" % tmp_xa2[2]
                        c=sam()
                        if tmp_xa2[1][0:1]=="+":
                        	c.strand=0
                        else :
                        	c.strand=1
                        c.chr=tmp_xa2[0]
                        c.pos=string.atoi(tmp_xa2[1][1:])
                        c.cigar=tmp_xa2[2]
	return c
def rc_seq(seq0):
	seq=seq0.upper()
        tmpseq=seq.replace("A","a")
	seq=tmpseq
        tmpseq=seq.replace("T","A")
	seq=tmpseq
        tmpseq=seq.replace("a","T")
	seq=tmpseq
	tmpseq=seq.replace("G","g")
	seq=tmpseq
        tmpseq=seq.replace("C","G")
	seq=tmpseq
        tmpseq=seq.replace("g","C")
	seq=tmpseq[::-1]
	return seq
def cal_mean_std(filename0,lower,upper,read_num):
	f1=maybe_gzip_open(filename0,'r')
	#f1=open(filename0,'r')
	#fout=open(filename+"_extend_difficult","w")
	tmp=filename0.split("/")
	filename=tmp[-1]
	#fout_single=open(out_folder+"/"+filename+"_single","w")
	#fout_del=open(out_folder+"/"+filename+"_large","w")
	#fout_inv=open(out_folder+"/"+filename+"_inver","w")
	#fout_dup=open(out_folder+"/"+filename+"_dup","w")
	n = 0
	tol_isize = 0
	std = 0
	isize_list=[]
	while True:
		if n == read_num:
			break
    		line1 = f1.readline()
		if line1[0:1]=='@':
			continue
    		if len(line1) == 0: # Zero length indicates EOF
        		break
    		line2 = f1.readline()
		if line2[0:1]=='@':
			print "line 2 contains @"
			sys.exit(1)	
    		if len(line2) == 0: # Zero length indicates EOF
			print "odd line num"
			sys.exit(1)	
		a = sam()
		b = sam()
		load_read(line1,a)
		load_read(line2,b)
		
		if a.name != b.name:
			print "file not in pair."
			sys.exit(1);
		if (a.unmap or a.difficult>0) or  ( b.unmap or b.difficult>0) or (abs(a.isize) >upper or abs(a.isize) < lower ):
                        continue
		n = n +1
		if n % 10000 == 0 and n > 0:
			print n 
		#print a.isize,b.isize
		isize_list.append(abs(a.isize))
	print "Average:\t", numpy.average(isize_list)
        print "Stddev:\t",numpy.std(isize_list)
	
	f1.close()


if __name__=='__main__':
	lower = 100
	upper = 1000
	read_num = 1000000
	if len(sys.argv)<2 or len(sys.argv)>5:
		usage()
		sys.exit(1)
	### bwa output sam file ###
	file1=sys.argv[1]
	for i in range(2,len(sys.argv)):
		if i==2:
			lower=string.atoi(sys.argv[i])
		if i==3:
			upper=string.atoi(sys.argv[i])
		if i==4:
			read_num = string.atoi(sys.argv[i])
	print lower,upper
	print read_num
	cal_mean_std(file1,lower,upper,read_num)
