#!/bin/bash

set -eu
set -o pipefail

if [ "$#" != "5" ] ;then
        echo "cnver_cluster.sh [mean of insert size] [std of insert size] [min pair number for a cluster] [input sam file prefix] [mem for sort]"
        exit 0
fi

export PRISM_PATH
mean=$1 #500
stdev=$2 #30 
tolerance=`expr $mean + 3 \* ${stdev}` 
min_mps=$3 #5
input_sam_prefix=$4
input_sam_path=$(dirname "${input_sam_prefix}")
sort_mem=$5
#input_sam_file=$5
export PRISM_PATH=$PRISM_PATH
if test ! -d ${PRISM_PATH} || [ -z ${PRISM_PATH} ];then
        echo  "PRISM_PATH(${PRISM_PATH}) is invalid."
        exit 0;
fi
if test ! -e "${PRISM_PATH}/bin/smapper";then
        echo "PRiSM bin files lost."
        exit 0
fi
if test ! -e "${PRISM_PATH}/bin/cluster_matepairs";then
        echo "Cluster bin files lost."
        exit 0
fi


cnver_path="${PRISM_PATH}/bin" #"/data/jiangyue/cnver-0.7.2"


echo "input samfile path:" $input_sam_path
echo "cnver path:" $cnver_path
echo "mean isize:" $mean
echo "isize stdev:" $stdev
echo "tolerance:" $tolerance
echo "min support reads num:" $min_mps

cat ${input_sam_prefix}_large |awk  -v a=$tolerance '{if(NR>0){if($9>a){printf("%s\t%s\t%s\t%s\t%s\t0\t+\t-\n",$9,$3,$4,$8,$1)}}}'>${input_sam_path}/test.disc.t0

awk -v type=0 -v findex=-1 '{  if (FNR == 1) findex++; $5 = findex "_" $5; if ($6 == type) print  findex "_" FNR-1 "_" $2,  $0}' ${input_sam_path}/test.disc.t0  | sort -k4n,4 -k5n,5 -k2n,2 -S ${sort_mem} | ${cnver_path}/cluster_matepairs --mean=${mean} --stdev=${stdev} --colID=0 --colDist=1 --colChr=2 --colLeft=3 --colRight=4 --colTemplate=5 --baseLenFactor=1 --mdJoinTolerance=6  --type=0 > ${input_sam_path}/clusters.t0 || true

cat ${input_sam_prefix}_dup |awk  -v a=0 '{if(NR>1){if($9>a){printf("%s\t%s\t%s\t%s\t%s\t1\t-\t+\n",$9,$3,$4,$8,$1)}}}' > ${input_sam_path}/test.disc.t1

awk -v type=1 -v findex=-1 '{  if (FNR == 1) findex++; $5 = findex "_" $5; if ($6 == type) print  findex "_" FNR-1 "_" $2,  $0}' ${input_sam_path}/test.disc.t1  | sort -k5n,5 -k4n,4 -k2n,2 -S ${sort_mem} | ${cnver_path}/cluster_matepairs --mean=${mean} --stdev=${stdev} --colID=0 --colDist=1 --colChr=2 --colLeft=3 --colRight=4 --colTemplate=5 --baseLenFactor=1 --mdJoinTolerance=6  --type=1 >  ${input_sam_path}/clusters.t1 || true
cat ${input_sam_prefix}_inver |awk  -v a=0 '{if(NR>1){if($9>a && and($2,0x10)==0x00){printf("%s\t%s\t%s\t%s\t%s\t2\t+\t+\n",$9,$3,$4,$8,$1)}}}'>${input_sam_path}/test.disc.t2

cat ${input_sam_prefix}_inver |awk  -v a=0 '{if(NR>1){if($9>a && and($2,0x10)==0x10){printf("%s\t%s\t%s\t%s\t%s\t3\t-\t-\n",$9,$3,$4,$8,$1)}}}'>${input_sam_path}/test.disc.t3
awk -v type=2 -v findex=-1 '{  if (FNR == 1) findex++; $5 = findex "_" $5; if ($6 == type) print  findex "_" FNR-1 "_" $2,  $0}' ${input_sam_path}/test.disc.t2  | sort -k5n,5 -k4n,4 -k2n,2 -S ${sort_mem} | ${cnver_path}/cluster_matepairs --mean=${mean} --stdev=${stdev} --colID=0 --colDist=1 --colChr=2 --colLeft=3 --colRight=4 --colTemplate=5 --baseLenFactor=1 --mdJoinTolerance=6  --type=2 >  ${input_sam_path}/clusters.t2 || true

awk -v type=3 -v findex=-1 '{  if (FNR == 1) findex++; $5 = findex "_" $5; if ($6 == type) print  findex "_" FNR-1 "_" $2,  $0}' ${input_sam_path}/test.disc.t3  | sort -k5n,5 -k4n,4 -k2n,2 -S ${sort_mem}| ${cnver_path}/cluster_matepairs --mean=${mean} --stdev=${stdev} --colID=0 --colDist=1 --colChr=2 --colLeft=3 --colRight=4 --colTemplate=5 --baseLenFactor=1 --mdJoinTolerance=6  --type=3 >  ${input_sam_path}/clusters.t3 || true


 grep -h EDGE ${input_sam_path}/clusters.t0 | ${cnver_path}/screen_contig_breaks --mean=${mean} --stdev=${stdev} --tolerance=${tolerance} --breaksFile=empty | awk -v min_mps=${min_mps} -v maxlen=10000000 '{ if (($5 >= min_mps) && ($8 < maxlen)) print $0 }' > ${input_sam_prefix}.links.t0.del  || ture
 grep -h EDGE ${input_sam_path}/clusters.t1 | ${cnver_path}/screen_contig_breaks --mean=${mean} --stdev=${stdev} --tolerance=${tolerance} --breaksFile=empty | awk -v min_mps=${min_mps} -v maxlen=10000000 '{ if (($5 >= min_mps) && ($8 < maxlen)) print $0 }' >  ${input_sam_prefix}.links.t1.dup || true
 grep -h EDGE ${input_sam_path}/clusters.t2 | ${cnver_path}/screen_contig_breaks --mean=${mean} --stdev=${stdev} --tolerance=${tolerance} --breaksFile=empty | awk -v min_mps=${min_mps} -v maxlen=10000000 '{ if (($5 >= min_mps) && ($8 < maxlen)) print $0 }' >  ${input_sam_prefix}.links.t2.inver_plus ||true
 grep -h EDGE ${input_sam_path}/clusters.t3 | ${cnver_path}/screen_contig_breaks --mean=${mean} --stdev=${stdev} --tolerance=${tolerance} --breaksFile=empty | awk -v min_mps=${min_mps} -v maxlen=10000000 '{ if (($5 >= min_mps) && ($8 < maxlen)) print $0 }' >  ${input_sam_prefix}.links.t3.inver_minus ||true

 #sort -k2,2n ${input_sam_prefix}.links.t0.indel |awk '($3-$2)>($8-500)'>  ${input_sam_prefix}.links.indel.fw
 #sort -k3,3n ${input_sam_prefix}.links.t0.indel |awk '($3-$2)>($8-500)'>  ${input_sam_prefix}.links.indel.bw
 #sort -k2,2n ${input_sam_prefix}.links.t1.dup |awk '($3-$2)>($8-500)'>  ${input_sam_prefix}.links.dup.fw
 #sort -k3,3n ${input_sam_prefix}.links.t1.dup |awk '($3-$2)>($8-500)'>  ${input_sam_prefix}.links.dup.bw
 #sort -k2,2n ${input_sam_prefix}.links.t2.inver_plus |awk '($3-$2)>($8-500)'>  ${input_sam_prefix}.links.inver.fw
 #sort -k3,3n ${input_sam_prefix}.links.t3.inver_minus |awk '($3-$2)>($8-500)'>  ${input_sam_prefix}.links.inver.bw
 sort -S ${sort_mem} -k2,2n ${input_sam_prefix}.links.t0.del >  ${input_sam_prefix}.links.del.fw
 sort -S ${sort_mem} -k3,3n ${input_sam_prefix}.links.t0.del >  ${input_sam_prefix}.links.del.bw
 sort -S ${sort_mem} -k2,2n ${input_sam_prefix}.links.t1.dup >  ${input_sam_prefix}.links.dup.fw
 sort -S ${sort_mem} -k3,3n ${input_sam_prefix}.links.t1.dup >  ${input_sam_prefix}.links.dup.bw
 sort -S ${sort_mem} -k2,2n ${input_sam_prefix}.links.t2.inver_plus >  ${input_sam_prefix}.links.inver.fw
 sort -S ${sort_mem} -k3,3n ${input_sam_prefix}.links.t3.inver_minus >  ${input_sam_prefix}.links.inver.bw
