#/usr/bin/env bash
set -eu
set -o pipefail

prism_sam="_none"
ref="_none"
supp_read_num=2
best_score_thresh=874
second_score_thresh=500
third_score_thresh=500
sort_mem="1g"

usage()
{
        echo "usage:"
        echo "  -i PRiSM output sam file."
        echo "  -r reference file"
        echo "  -S mem for sort[default:1g]"
        exit 0
}

export PRISM_PATH

if test ! -d ${PRISM_PATH} || [ -z ${PRISM_PATH} ];then
        echo  "PRISM_PATH(${PRISM_PATH}) is invalid."
        exit 0;
fi
if test ! -e "${PRISM_PATH}/bin/smapper";then
        echo "PRiSM bin files lost."
        exit 0
fi
if test ! -e "${PRISM_PATH}/bin/cluster_matepairs";then
        echo "Cluster bin files lost."
        exit 0
fi

while getopts "i:n:1:2:3:S:r:" options;do
        case $options in
                i)      prism_sam=$OPTARG;echo -e "PRiSM sam:\t${prism_sam}";;
                n)      supp_read_num=$OPTARG;echo -e "Support read num:\t${supp_read_num}";;
                1)      best_score_thresh=$OPTARG;echo -e "1st score thresh:\t${best_score_thresh}";;
                2)      second_score_thresh=$OPTARG;echo -e "2st score thresh:\t${second_score_thresh}";;
                3)      third_score_thresh=$OPTARG;echo -e "3st score thresh:\t${third_score_thresh}";;
		S)	sort_mem=$OPTARG;;
		r)	ref=$OPTARG;;
                \?)     usage;;
        esac
done
if test ! -e "${PRISM_PATH}/bin/smapper";then
        echo "PRiSM bin files lost."
        #exit 0
fi

if test ! -e ${prism_sam};then
        echo "input file is invalid:$prism_sam."
        usage
        exit 0
fi
if test ! -e ${ref};then
        echo "reference file is invalid:$ref."
        usage
        exit 0
fi

output_path=$(dirname "${prism_sam}")
sort -S ${sort_mem} -k1,1 ${prism_sam} -o ${prism_sam}_ns
${PRISM_PATH}/toolkit/rm_multi_read.py ${prism_sam}_ns
#${PRISM_PATH}/bin/smapper formatsv ${prism_sam}_rmmul 
${PRISM_PATH}/toolkit/format_sv.py ${prism_sam}_ns_rmmul 
sort -S ${sort_mem} -k2,2n -k3,3n -k6,6 ${prism_sam}_ns_rmmul_cigar -o ${prism_sam}_ns_rmmul_cigar_sorted
${PRISM_PATH}/toolkit/detect_sv.py ${prism_sam}_ns_rmmul_cigar_sorted ${ref}

${PRISM_PATH}/toolkit/filter.sh -i ${prism_sam}_ns_rmmul_cigar_sorted_sv -o ${output_path}

#${PRISM_PATH}/toolkit/rm_overlap_sv.py ${output_path}/del_50_100
#${PRISM_PATH}/toolkit/rm_overlap_sv.py ${output_path}/del_100_1000
#${PRISM_PATH}/toolkit/rm_overlap_sv.py ${output_path}/del_1000_plus
#${PRISM_PATH}/toolkit/rm_overlap_sv.py ${output_path}/dup
#${PRISM_PATH}/toolkit/rm_overlap_sv.py ${output_path}/inv

rm ${prism_sam}_ns_rmmul_cigar
#rm ${prism_sam}_ns_rmmul_cigar_sorted
#rm ${output_path}/del_100_1000
#rm ${output_path}/del_1000_plus
#rm ${output_path}/dup
#rm ${output_path}/inv
