/*	$Id: fasta.c,v 1.23 2009/06/16 23:26:21 rumble Exp $	*/

#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <zlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include "../common/fasta.h"
#include "../common/util.h"

static uint64_t total_ticks;

fasta_t
fasta_open(const char *file, int space)
{
	fasta_t fasta = NULL;
	struct stat sb;
	gzFile fp;
	uint64_t before = rdtsc();

	assert(space == COLOUR_SPACE || space == LETTER_SPACE);

	if (stat(file, &sb))
		goto out;

	if (!S_ISREG(sb.st_mode))
		goto out;

	fp = gzopen(file, "r");
	if (fp == NULL)
		goto out;

	fasta = (fasta_t)xmalloc(sizeof(*fasta));
	memset(fasta, 0, sizeof(*fasta));

	fasta->fp = fp;
	fasta->file = xstrdup(file);
	fasta->space = space;
	memset(fasta->translate, -1, sizeof(fasta->translate));

	if (space == COLOUR_SPACE) {
		fasta->translate[(int)'0'] = BASE_0;
		fasta->translate[(int)'1'] = BASE_1;
		fasta->translate[(int)'2'] = BASE_2;
		fasta->translate[(int)'3'] = BASE_3;
		fasta->translate[(int)'4'] = BASE_N;
		fasta->translate[(int)'N'] = BASE_N;
		fasta->translate[(int)'n'] = BASE_N;
		fasta->translate[(int)'.'] = BASE_N;
		fasta->translate[(int)'X'] = BASE_X;
		fasta->translate[(int)'x'] = BASE_X;
	} else {	
		fasta->translate[(int)'A'] = BASE_A;
		fasta->translate[(int)'a'] = BASE_A;
		fasta->translate[(int)'C'] = BASE_C;
		fasta->translate[(int)'c'] = BASE_C;
		fasta->translate[(int)'G'] = BASE_G;
		fasta->translate[(int)'g'] = BASE_G;
		fasta->translate[(int)'T'] = BASE_T;
		fasta->translate[(int)'t'] = BASE_T;
		fasta->translate[(int)'U'] = BASE_U;
		fasta->translate[(int)'u'] = BASE_U;
		fasta->translate[(int)'M'] = BASE_M;
		fasta->translate[(int)'m'] = BASE_M;
		fasta->translate[(int)'R'] = BASE_R;
		fasta->translate[(int)'r'] = BASE_R;
		fasta->translate[(int)'W'] = BASE_W;
		fasta->translate[(int)'w'] = BASE_W;
		fasta->translate[(int)'S'] = BASE_S;
		fasta->translate[(int)'s'] = BASE_S;
		fasta->translate[(int)'Y'] = BASE_Y;
		fasta->translate[(int)'y'] = BASE_Y;
		fasta->translate[(int)'K'] = BASE_K;
		fasta->translate[(int)'k'] = BASE_K;
		fasta->translate[(int)'V'] = BASE_V;
		fasta->translate[(int)'v'] = BASE_V;
		fasta->translate[(int)'H'] = BASE_H;
		fasta->translate[(int)'h'] = BASE_H;
		fasta->translate[(int)'D'] = BASE_D;
		fasta->translate[(int)'d'] = BASE_D;
		fasta->translate[(int)'B'] = BASE_B;
		fasta->translate[(int)'b'] = BASE_B;
		fasta->translate[(int)'N'] = BASE_N;
		fasta->translate[(int)'n'] = BASE_N;
		fasta->translate[(int)'.'] = BASE_N;
		fasta->translate[(int)'X'] = BASE_X;
		fasta->translate[(int)'x'] = BASE_X;
	}

 out:
	total_ticks += (rdtsc() - before);
	return (fasta);
}

fasta_t
fasta_open2(int space)
{
	fasta_t fasta = NULL;


	uint64_t before = rdtsc();

	assert(space == COLOUR_SPACE || space == LETTER_SPACE);


	fasta = (fasta_t)xmalloc(sizeof(*fasta));
	memset(fasta, 0, sizeof(*fasta));


	fasta->space = space;
	memset(fasta->translate, -1, sizeof(fasta->translate));

	if (space == COLOUR_SPACE) {
		fasta->translate[(int)'0'] = BASE_0;
		fasta->translate[(int)'1'] = BASE_1;
		fasta->translate[(int)'2'] = BASE_2;
		fasta->translate[(int)'3'] = BASE_3;
		fasta->translate[(int)'4'] = BASE_N;
		fasta->translate[(int)'N'] = BASE_N;
		fasta->translate[(int)'n'] = BASE_N;
		fasta->translate[(int)'.'] = BASE_N;
		fasta->translate[(int)'X'] = BASE_X;
		fasta->translate[(int)'x'] = BASE_X;
	} else {
		fasta->translate[(int)'A'] = BASE_A;
		fasta->translate[(int)'a'] = BASE_A;
		fasta->translate[(int)'C'] = BASE_C;
		fasta->translate[(int)'c'] = BASE_C;
		fasta->translate[(int)'G'] = BASE_G;
		fasta->translate[(int)'g'] = BASE_G;
		fasta->translate[(int)'T'] = BASE_T;
		fasta->translate[(int)'t'] = BASE_T;
		fasta->translate[(int)'U'] = BASE_U;
		fasta->translate[(int)'u'] = BASE_U;
		fasta->translate[(int)'M'] = BASE_M;
		fasta->translate[(int)'m'] = BASE_M;
		fasta->translate[(int)'R'] = BASE_R;
		fasta->translate[(int)'r'] = BASE_R;
		fasta->translate[(int)'W'] = BASE_W;
		fasta->translate[(int)'w'] = BASE_W;
		fasta->translate[(int)'S'] = BASE_S;
		fasta->translate[(int)'s'] = BASE_S;
		fasta->translate[(int)'Y'] = BASE_Y;
		fasta->translate[(int)'y'] = BASE_Y;
		fasta->translate[(int)'K'] = BASE_K;
		fasta->translate[(int)'k'] = BASE_K;
		fasta->translate[(int)'V'] = BASE_V;
		fasta->translate[(int)'v'] = BASE_V;
		fasta->translate[(int)'H'] = BASE_H;
		fasta->translate[(int)'h'] = BASE_H;
		fasta->translate[(int)'D'] = BASE_D;
		fasta->translate[(int)'d'] = BASE_D;
		fasta->translate[(int)'B'] = BASE_B;
		fasta->translate[(int)'b'] = BASE_B;
		fasta->translate[(int)'N'] = BASE_N;
		fasta->translate[(int)'n'] = BASE_N;
		fasta->translate[(int)'.'] = BASE_N;
		fasta->translate[(int)'X'] = BASE_X;
		fasta->translate[(int)'x'] = BASE_X;
	}


	total_ticks += (rdtsc() - before);
	return (fasta);
}

void
fasta_close(fasta_t fasta)
{
	uint64_t before = rdtsc();

	gzclose(fasta->fp);
	free(fasta->file);
	free(fasta);

	total_ticks += (rdtsc() - before);
}

fasta_stats_t
fasta_stats()
{
	fasta_stats_t fs;

	fs = (fasta_stats_t)xmalloc(sizeof(*fs));
	fs->total_ticks = total_ticks;

	return (fs);
}

static char *
extract_name(char *buffer, char * * ranges)
{
	char *extracted;
	char *ret;
	int len;
	char * tok_save;

	assert(buffer[0] == '>');

	/* Funny business for valgrind. See bottom of fasta_get_next. */
	/*
	extracted = strtrim(&buffer[1]);
	len = strlen(extracted);
	ret = (char *)xmalloc(len + 17);
	memcpy(ret, extracted, len);
	memset(ret + len, 0, 17);
	*/

	extracted = strtok_r(&buffer[1], "\t", &tok_save);
	extracted = strtrim(extracted);
	len = strlen(extracted);
	ret = (char *)xmalloc(len + 17);
	memcpy(ret, extracted, len);
	memset(ret + len, 0, 17);

	if (ranges != NULL && (extracted = strtok_r(NULL, "\t", &tok_save)) != NULL) {
	  len = strlen(extracted);
	  *ranges = (char *)xmalloc(len + 17);
	  memcpy(*ranges, extracted, len);
	  memset(*ranges + len, 0, 17);
	}

	return (ret);
}

bool
fasta_get_next_with_range(fasta_t fasta, char **name, char **sequence, bool *is_rna, char * * ranges)
{
	static int categories[256] = { 42 };
	enum { CAT_SPACE, CAT_NEWLINE, CAT_NUL, CAT_ELSE };

	int i;
	bool gotname = false;
	uint32_t sequence_length = 0;
	uint32_t max_sequence_length = 0;
	uint64_t before = rdtsc();
	char *readinseq;

	/* isspace(3) is really slow, so build a lookup instead */
	if (categories[0] == 42) {
		for (i = 0; i < 256; i++) {
			if (i == '\n')
				categories[i] = CAT_NEWLINE;
			else if (i == '\0')
				categories[i] = CAT_NUL;
			else if (isspace(i))
				categories[i] = CAT_SPACE;
			else if (i == '-')	/* simply ignore dashes in haplome alignments */
				categories[i] = CAT_SPACE;
			else
				categories[i] = CAT_ELSE;
		}
	}

	*name = *sequence = readinseq = NULL;

	if (fasta->leftover) {
		*name = extract_name(fasta->buffer, ranges);
		fasta->leftover = false;
		gotname = true;
	}

	while (fast_gzgets(fasta->fp, fasta->buffer, sizeof(fasta->buffer)) != NULL) {
		if (fasta->buffer[0] == '#')
			continue;

		if (fasta->buffer[0] == '>') {
			if (gotname) {
				fasta->leftover = true;
				break;
			}

			*name = extract_name(fasta->buffer, ranges);
			gotname = true;
			continue;
		}

		if (max_sequence_length == 0 ||
		    max_sequence_length - sequence_length <= sizeof(fasta->buffer)) {
			if (max_sequence_length == 0)
				max_sequence_length = 16*1024*1024;
			else if (max_sequence_length >= 128*1024*1024)
				max_sequence_length += 128*1024*1024;
			else
				max_sequence_length *= 2;
			readinseq = (char *)xrealloc(readinseq, max_sequence_length);
		}

		for (i = 0; fasta->buffer[i] != '\0'; i++) {
			int act = categories[(int)fasta->buffer[i]];

			assert(i < (int)sizeof(fasta->buffer));

			switch (act) {
			case CAT_NUL:
			case CAT_NEWLINE:
				fasta->buffer[i] = '\0';
				goto out;
			case CAT_SPACE:
				continue;
			case CAT_ELSE:
			default:
				/* fall down below */
				break;
			}

			readinseq[sequence_length++] = fasta->buffer[i];

			continue;
 out:
			break;
		}

		assert(sequence_length <= max_sequence_length);
	}

	if (!gotname) {
		if (readinseq != NULL)
			free(readinseq);
		total_ticks += (rdtsc() - before);
		return (false);
	}

	/* if we read in a tag name but there's no sequence, don't just return false */ 
	if (gotname && readinseq == NULL) {
		readinseq = xstrdup("");
		sequence_length = 1;
	}

	if (readinseq == NULL)
		return (false);

	/*
	 * Ensure nul-termination and allocate extra space to appease valgrind.
	 * I think strlen may do > char-sized loads and valgrind thinks it's
	 * conditionally jumping on uninitialised memory. That explanation
	 * doesn't seem right to me, but doing this makes it happy again.
	 */
	*sequence = (char *)xmalloc(sequence_length + 17);
	memcpy(*sequence, readinseq, sequence_length);
	memset(*sequence + sequence_length, 0, 17);

	/* check if the sequence is rna (contains uracil and not thymine) */
	if (is_rna != NULL) {
		bool got_uracil = false;
		bool got_thymine = false;
		unsigned int j;

		for (j = 0; j < sequence_length; j++) {
			unsigned char chr = readinseq[j];
			got_thymine |= (chr == 'T' || chr == 't');
			got_uracil  |= (chr == 'U' || chr == 'u');
		}

		if (got_uracil && got_thymine)
			fprintf(stderr, "WARNING: sequence has both uracil and "
			    "thymine!?!\n");

		*is_rna = (got_uracil && !got_thymine);
	}

	free(readinseq);
	assert(*name != NULL);
	assert(*sequence != NULL);
	total_ticks += (rdtsc() - before);
	return (true);
}

int
fasta_get_initial_base(fasta_t fasta, char *sequence)
{

	assert(fasta->space == COLOUR_SPACE);

	switch (*sequence) {
	case 'A':
	case 'a':
		return (BASE_A);
	case 'C':
	case 'c':
		return (BASE_C);
	case 'G':
	case 'g':
		return (BASE_G);
	case 'T':
	case 't':
		return (BASE_T);
	}

	return (-1);
}

uint32_t *
fasta_bitfield_to_colourspace(fasta_t fasta, uint32_t *source, uint32_t length, bool is_rna)
{
	int a, lastbp = BASE_T;
	uint32_t *dst;
	uint32_t i;
	uint64_t before = rdtsc();

	assert(fasta->space == LETTER_SPACE);

	dst = (uint32_t *)xmalloc(BPTO32BW(length) * sizeof(uint32_t));
	memset(dst, 0, BPTO32BW(length) * sizeof(uint32_t));

	for (i = 0; i < length; i++) {
		a = EXTRACT(source, i);
		bitfield_insert(dst, i, lstocs(lastbp, a, is_rna));
		lastbp = a;
	}

	total_ticks += (rdtsc() - before);
	return (dst);
}

uint32_t *
fasta_sequence_to_bitfield(fasta_t fasta, char *sequence)
{
	uint32_t i, length, idx;
	uint32_t *bitfield;
	uint64_t before = rdtsc();
	int a;
	char c;

	length = strlen(sequence);
	bitfield = (uint32_t *)xmalloc(BPTO32BW(length) * sizeof(uint32_t));
	memset(bitfield, 0, BPTO32BW(length) * sizeof(uint32_t));

	i = 0;

	c = (char)sequence[0];

	if (fasta->space == COLOUR_SPACE) {
		if (c != 'A' && c != 'a' &&
		    c != 'C' && c != 'c' && 
		    c != 'G' && c != 'g' &&
		    c != 'T' && c != 't') {
			free(bitfield);
			total_ticks += (rdtsc() - before);
			return (NULL);
		}
		
		i = 1;
	}

	for (idx = 0; i < length; i++) {
		a = fasta->translate[(int)sequence[i]];//BASEA, BASEC, ...

		if (a == -1) {
			fprintf(stderr, "error: invalid character ");
			uint32_t k;
			for(k=0;k<length;k++){
				fprintf(stderr,"%c",sequence[k]);
			}
			fprintf(stderr,"\n");
			if (isprint(a))
				fprintf(stderr, "(%c) ", a);
			else if (a != -1)
				fprintf(stderr, "(0x%x) ", a);
			fprintf(stderr, "in input file [%s]\n", fasta->file);
			fprintf(stderr, "       (Did you mix up letter "
			    "space and colour space programs?)\n");
			exit(1);
		}

		if (fasta->space == COLOUR_SPACE) {
			assert((a >= BASE_CS_MIN && a <= BASE_CS_MAX) ||
			    (a == BASE_N || a == BASE_X));
		} else {
			assert((a >= BASE_LS_MIN && a <= BASE_LS_MAX) ||
			    (a == BASE_N || a == BASE_X));
		}

		bitfield_append(bitfield, idx++, a);
	}

	if (fasta->space == COLOUR_SPACE)
		assert(idx == length - 1);
	else
		assert(idx == length);

	total_ticks += (rdtsc() - before);
	return (bitfield);
}

/*
 * Give BASE_x, return the appropriate character.
 *
 * NB: Since we're limited to 4-bits, BASE_X returns 'N'.
 */
char
base_translate(int base, bool use_colours)
{
	/*
	 * NB: colour-space only valid for 0-3 and BASE_N/BASE_X
	 *     BASE_N is reported as a skipped cycle: '.' in CS.
	 */
	char cstrans[] = { '0', '1', '2', '3', '!', '@', '#', '$',
			   '%', '^', '&', '*', '?', '~', ';', '.' };
	char lstrans[] = { 'A', 'C', 'G', 'T', 'U', 'M', 'R', 'W',
			   'S', 'Y', 'K', 'V', 'H', 'D', 'B', 'N' };

	if (use_colours) {
		assert((base >= BASE_CS_MIN && base <= BASE_CS_MAX) ||
		    (base == BASE_N || base == BASE_X));
		return (cstrans[base]);
	} else {
		assert((base >= BASE_LS_MIN && base <= BASE_LS_MAX) ||
		    (base == BASE_N || base == BASE_X));
		return (lstrans[base]);
	}
}

/**
 * read sam file
 */
int
read_sam(sam_t sam){//, char **readname, int **mpos){

	if(gzgets(sam->fp,sam->buffer1,1024)!=NULL){
		//printf("sam string:%s\n",sam->buffer);
		return 1;
	}else{
		return 0;
	}


}
/**
 * read a pair of lines from sam file at one time
 */
int
pair_read_sam_gz(sam_t sam1,gzFile file){//, char **readname, int **mpos){

	while(gzgets(sam1->fp,sam1->buffer1,1024)!=NULL && sam1->buffer1[0]=='@'){
		printf("%s",sam1->buffer1);
	}
	if(gzgets(sam1->fp,sam1->buffer2,1024)!=NULL){
		//printf("sam string1:%s\n",sam1->buffer);
		//printf("sam string2:%s\n",sam2->buffer);
		//FILE *file=fopen("paired.sam","at");

		char *sam_str1[50], *sam_str2[50];
		memset(sam_str1,0,50);
		memset(sam_str2,0,50);
		char *tok;
		int i1,i2,i;

		//read each field in sam file
		i2=0;

		tok=strtok(sam1->buffer2,"\t");

		while(tok!=NULL){
			sam_str2[i2]=xstrdup(tok);
			tok = strtok (NULL, "\t");
			i2++;
		}
		i1=0;
		tok=strtok(sam1->buffer1,"\t");
		while(tok!=NULL){
			sam_str1[i1]=xstrdup(tok);
			tok = strtok (NULL, "\t");
			i1++;
		}

		for(i=0;i<i1-1;i++){
			if(i==7){
				gzputs(file, sam_str2[3]);
				gzputs(file, "\t");
			}else{
				gzputs(file, sam_str1[i]);
				gzputs(file, "\t");
				//fprintf(file,"%s\t",sam_str1[i]);
			}
		}
		tok=strtok(sam_str1[i1-1],"\n");
		gzputs(file, sam_str1[i1-1]);
		gzputs(file, "\t");
		gzputs(file, "XS:Z:");
		gzputs(file, sam_str2[9]);
		gzputs(file, "\n");
		//fprintf(file,"%s\t",sam_str1[i1-1]);
		//fprintf(file,"XS:Z:%s\n",sam_str2[9]);

		for(i=0;i<i2-1;i++){
			if(i==7){
				gzputs(file, sam_str1[3]);
				gzputs(file, "\t");
				//fprintf(file,"%s\t",sam_str1[3]);
			}else{
				gzputs(file, sam_str2[i]);
				gzputs(file, "\t");
				//fprintf(file,"%s\t",sam_str2[i]);
			}
		}
		tok=strtok(sam_str2[i2-1],"\n");
		gzputs(file, sam_str2[i2-1]);
		gzputs(file, "\t");
		gzputs(file, "XS:Z:");
		gzputs(file, sam_str1[9]);
		gzputs(file, "\n");
		//fprintf(file,"%s\t",sam_str2[i2-1]);
		//fprintf(file,"XS:Z:%s\n",sam_str1[9]);

		//fclose(file);


		for(i=0;i<i1;i++){
			if(sam_str1[i]!=NULL){
				free(sam_str1[i]);
			}

		}
		for(i=0;i<i2;i++){
			if(sam_str2[i]!=NULL){
				free(sam_str2[i]);
			}

		}
		return 1;
	}else{
		return 0;
	}


}
/**
 * read a pair of lines from sam file at one time
 */
int
pair_read_sam(sam_t sam1,FILE *file){//, char **readname, int **mpos){
	char tmp_buffer[1024];
	while(gzgets(sam1->fp,sam1->buffer1,1024)!=NULL && sam1->buffer1[0]=='@'){
		printf("%s",sam1->buffer1);
	}
	if(gzgets(sam1->fp,sam1->buffer2,1024)!=NULL){
		//printf("sam string1:%s\n",sam1->buffer);
		//printf("sam string2:%s\n",sam2->buffer);
		//FILE *file=fopen("paired.sam","at");

		char *sam_str1[50], *sam_str2[50];
		memset(sam_str1,0,50);
		memset(sam_str2,0,50);
		char *tok;
		int i1,i2,i;

		//read each field in sam file
		i2=0;

		strcpy(tmp_buffer,sam1->buffer2);
		//tok=strtok(sam1->buffer2,"\t");
		tok=strtok(tmp_buffer,"\t");
		while(tok!=NULL){
			sam_str2[i2]=xstrdup(tok);
			tok = strtok (NULL, "\t");
			i2++;
		}
		i1=0;
		strcpy(tmp_buffer,sam1->buffer1);
		//tok=strtok(sam1->buffer1,"\t");
		tok=strtok(tmp_buffer,"\t");
		while(tok!=NULL){
			sam_str1[i1]=xstrdup(tok);
			tok = strtok (NULL, "\t");
			i1++;
		}

		if((atoi(sam_str2[1])&0x4)==0x4 && (atoi(sam_str1[1])&0x4)==0x0){//read #1 mapped
			for(i=0;i<i1-1;i++){
				if(i==6){
					fprintf(file,"%s\t",sam_str2[2]);
				}else if(i==7){
					//gzputs(file, sam_str2[3]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str2[3]);
				}else{
					//gzputs(file, sam_str1[i]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str1[i]);
				}
			}
			tok=strtok(sam_str1[i1-1],"\n");
			//gzputs(file, sam_str1[i1-1]);
			//gzputs(file, "\t");
			//gzputs(file, "XS:Z:");
			//gzputs(file, sam_str2[9]);
			//gzputs(file, "\n");
			fprintf(file,"%s\t",sam_str1[i1-1]);
			fprintf(file,"XS:Z:%s\n",sam_str2[9]);
		}else if((atoi(sam_str1[1])&0x4)==0x4 && (atoi(sam_str2[1])&0x4)==0x0){//read #2 mapped
			for(i=0;i<i2-1;i++){
				if(i==6){
					fprintf(file,"%s\t",sam_str1[2]);
				}else if(i==7){
					//gzputs(file, sam_str1[3]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str1[3]);
				}else{
					//gzputs(file, sam_str2[i]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str2[i]);
				}
			}
			tok=strtok(sam_str2[i2-1],"\n");
			//gzputs(file, sam_str2[i2-1]);
			//gzputs(file, "\t");
			//gzputs(file, "XS:Z:");
			//gzputs(file, sam_str1[9]);
			//gzputs(file, "\n");
			fprintf(file,"%s\t",sam_str2[i2-1]);
			fprintf(file,"XS:Z:%s\n",sam_str1[9]);

			//fclose(file);




		}else if((atoi(sam_str1[1])&0x4)==0x0 && (atoi(sam_str2[1])&0x4)==0x0){//#1 & #2 mapped
			for(i=0;i<i1-1;i++){
				if(i==6){
					fprintf(file,"%s\t",sam_str2[2]);
				}else if(i==7){
					//gzputs(file, sam_str2[3]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str2[3]);
				}else{
					//gzputs(file, sam_str1[i]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str1[i]);
				}
			}
			tok=strtok(sam_str1[i1-1],"\n");
			//gzputs(file, sam_str1[i1-1]);
			//gzputs(file, "\t");
			//gzputs(file, "XS:Z:");
			//gzputs(file, sam_str2[9]);
			//gzputs(file, "\n");
			fprintf(file,"%s\n",sam_str1[i1-1]);
			//fprintf(file,"XS:Z:%s\n",sam_str2[9]);

			for(i=0;i<i2-1;i++){
				if(i==6){
					fprintf(file,"%s\t",sam_str1[2]);
				}else if(i==7){
					//gzputs(file, sam_str1[3]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str1[3]);
				}else{
					//gzputs(file, sam_str2[i]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str2[i]);
				}
			}
			tok=strtok(sam_str2[i2-1],"\n");
			//gzputs(file, sam_str2[i2-1]);
			//gzputs(file, "\t");
			//gzputs(file, "XS:Z:");
			//gzputs(file, sam_str1[9]);
			//gzputs(file, "\n");
			fprintf(file,"%s\n",sam_str2[i2-1]);
			//fprintf(file,"XS:Z:%s\n",sam_str1[9]);
		}else{

		}
		//free sam_str1 & sam_str2
		for(i=0;i<i1;i++){
			if(sam_str1[i]!=NULL){
				free(sam_str1[i]);
			}

		}
		for(i=0;i<i2;i++){
			if(sam_str2[i]!=NULL){
				free(sam_str2[i]);
			}

		}
		return 1;
	}else{
		return 0;
	}


}
sam_t
sam_open(const char *file)
{
	sam_t sam = NULL;

	gzFile fp;
	uint64_t before = rdtsc();
	if(file==NULL){
		goto out;
	}
	fp = gzopen(file, "r");

	if (fp == NULL){
		goto out;
	}

	sam = (sam_t)xmalloc(sizeof(*sam));
	memset(sam, 0, sizeof(*sam));

	int i;
	for(i=0;i<50;i++){
		sam->sam_field[i]=(char*)xcalloc(1024*sizeof(char));
	}

	sam->fp = fp;
	sam->file = xstrdup(file);

 out:
	total_ticks += (rdtsc() - before);

	return (sam);

}
void
sam_close(sam_t sam)
{
	uint64_t before = rdtsc();

	gzclose(sam->fp);
	free(sam->file);
	free(sam);

	total_ticks += (rdtsc() - before);
}
int
paired_sam(char *sam_file, char *pair_sam_file ){
	printf("test 2.2\n");
	sam_t sam1;
	sam1= sam_open(sam_file);

	//gzFile pfile=gzopen(pair_sam_file,"w");
	FILE *pfile=fopen(pair_sam_file,"w");
	if(pfile==NULL){
		fprintf(stderr,"file open failed");
		assert(0);
	}
	while(pair_read_sam(sam1, pfile)){

	}

	sam_close(sam1);
	fclose(pfile);
	//gzclose(pfile);
	return 1;
}
int
trim_sam0(char *sam_file, char *trimed_file){
	sam_t sam1;
	sam1= sam_open(sam_file);
	char *name;
	int flag;
	char tmp_buffer[1024];
	//gzFile pfile=gzopen(pair_sam_file,"w");
	FILE *pfile=fopen(trimed_file,"w");
	if(pfile==NULL){
		fprintf(stderr,"file open failed");
		assert(0);
	}
	while(gzgets(sam1->fp,sam1->buffer1,1024)!=NULL){
		if(sam1->buffer1[0]=='@'){//head
			continue;
		}
		strcpy(tmp_buffer, sam1->buffer1);
		name=strtok(tmp_buffer,"\t");
		flag=atoi(strtok(NULL,"\t"));
		printf("sam_buffer1:%s\n",sam1->buffer1);
		if((flag & 0x04) == 0x04){//not mapped
			continue;
		}
		fprintf(pfile,"%s\n",sam1->buffer1);

	}

	sam_close(sam1);
	fclose(pfile);
	//gzclose(pfile);
	return 1;
}


int
fix_mate(sam_t sam1,FILE *file){//, char **readname, int **mpos){
	char tmp_buffer[1024];
	while(gzgets(sam1->fp,sam1->buffer1,1024)!=NULL && sam1->buffer1[0]=='@'){
		printf("%s",sam1->buffer1);
	}
	if(gzgets(sam1->fp,sam1->buffer2,1024)!=NULL){
		//printf("sam string1:%s\n",sam1->buffer);
		//printf("sam string2:%s\n",sam2->buffer);
		//FILE *file=fopen("paired.sam","at");

		char *sam_str1[50], *sam_str2[50];
		memset(sam_str1,0,50);
		memset(sam_str2,0,50);
		char *tok;
		int i1,i2,i;

		//read each field in sam file
		i2=0;

		strcpy(tmp_buffer,sam1->buffer2);
		//tok=strtok(sam1->buffer2,"\t");
		tok=strtok(tmp_buffer,"\t");
		while(tok!=NULL){
			sam_str2[i2]=xstrdup(tok);
			tok = strtok (NULL, "\t");
			i2++;
		}
		i1=0;
		strcpy(tmp_buffer,sam1->buffer1);
		//tok=strtok(sam1->buffer1,"\t");
		tok=strtok(tmp_buffer,"\t");
		while(tok!=NULL){
			sam_str1[i1]=xstrdup(tok);
			tok = strtok (NULL, "\t");
			i1++;
		}

		if((atoi(sam_str2[1])&0x4)==0x4 && (atoi(sam_str1[1])&0x4)==0x0){//read #1 mapped
			for(i=0;i<i1-1;i++){
				if(i==6){
					fprintf(file,"%s\t",sam_str2[2]);
				}else if(i==7){
					//gzputs(file, sam_str2[3]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str2[3]);
				}else{
					//gzputs(file, sam_str1[i]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str1[i]);
				}
			}
			tok=strtok(sam_str1[i1-1],"\n");
			//gzputs(file, sam_str1[i1-1]);
			//gzputs(file, "\t");
			//gzputs(file, "XS:Z:");
			//gzputs(file, sam_str2[9]);
			//gzputs(file, "\n");
			fprintf(file,"%s\t",sam_str1[i1-1]);
			fprintf(file,"XS:Z:%s\n",sam_str2[9]);
		}else if((atoi(sam_str1[1])&0x4)==0x4 && (atoi(sam_str2[1])&0x4)==0x0){//read #2 mapped
			for(i=0;i<i2-1;i++){
				if(i==6){
					fprintf(file,"%s\t",sam_str1[2]);
				}else if(i==7){
					//gzputs(file, sam_str1[3]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str1[3]);
				}else{
					//gzputs(file, sam_str2[i]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str2[i]);
				}
			}
			tok=strtok(sam_str2[i2-1],"\n");
			//gzputs(file, sam_str2[i2-1]);
			//gzputs(file, "\t");
			//gzputs(file, "XS:Z:");
			//gzputs(file, sam_str1[9]);
			//gzputs(file, "\n");
			fprintf(file,"%s\t",sam_str2[i2-1]);
			fprintf(file,"XS:Z:%s\n",sam_str1[9]);

			//fclose(file);




		}else if((atoi(sam_str1[1])&0x4)==0x0 && (atoi(sam_str2[1])&0x4)==0x0){//#1 & #2 mapped
			for(i=0;i<i1-1;i++){
				if(i==6){
					fprintf(file,"%s\t",sam_str2[2]);
				}else if(i==7){
					//gzputs(file, sam_str2[3]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str2[3]);
				}else{
					//gzputs(file, sam_str1[i]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str1[i]);
				}
			}
			tok=strtok(sam_str1[i1-1],"\n");
			//gzputs(file, sam_str1[i1-1]);
			//gzputs(file, "\t");
			//gzputs(file, "XS:Z:");
			//gzputs(file, sam_str2[9]);
			//gzputs(file, "\n");
			fprintf(file,"%s\n",sam_str1[i1-1]);
			//fprintf(file,"XS:Z:%s\n",sam_str2[9]);

			for(i=0;i<i2-1;i++){
				if(i==6){
					fprintf(file,"%s\t",sam_str1[2]);
				}else if(i==7){
					//gzputs(file, sam_str1[3]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str1[3]);
				}else{
					//gzputs(file, sam_str2[i]);
					//gzputs(file, "\t");
					fprintf(file,"%s\t",sam_str2[i]);
				}
			}
			tok=strtok(sam_str2[i2-1],"\n");
			//gzputs(file, sam_str2[i2-1]);
			//gzputs(file, "\t");
			//gzputs(file, "XS:Z:");
			//gzputs(file, sam_str1[9]);
			//gzputs(file, "\n");
			fprintf(file,"%s\n",sam_str2[i2-1]);
			//fprintf(file,"XS:Z:%s\n",sam_str1[9]);
		}else{

		}
		//free sam_str1 & sam_str2
		for(i=0;i<i1;i++){
			if(sam_str1[i]!=NULL){
				free(sam_str1[i]);
			}

		}
		for(i=0;i<i2;i++){
			if(sam_str2[i]!=NULL){
				free(sam_str2[i]);
			}

		}
		return 1;
	}else{
		return 0;
	}


}



/*********************merge*****************************/

static int n_unique, n_fixed, n_over_mapped,n_low_score_u, n_all_u,n_low_score_m, n_pair_u, n_single_u,n_pair_m,n_single_m, n_fixed_u,n_fixed_m;

struct sam_field_t {
	int file_id;
	char **sam_field;//just keep the sam field of buffer[0], for compare only
	int field_num;
	char **buffer;
	int buffer_len;
	int buffer_num;
};
struct mapping_pos_t{
	char chr_name[1024];
	int pos;
	int ascore;

	int sft_id;
	int buffer_id;

	int flag;
};

int
parse_sam_field(char *buffer, char **sam_field, int *field_num){
	char *tok;
	int i;
	char *tmp_buffer;
	tmp_buffer=xstrdup(buffer);
	//strcpy(tmp_buffer,buffer);
	tok=strtok(tmp_buffer,"\t");
	i=0;
	while(tok!=NULL){
		strcpy(sam_field[i],tok);
		tok = strtok (NULL, "\t");
		i++;
	}
	*field_num=i;
	free(tmp_buffer);
	return 0;
}
int
fix_and_filter(struct sam_field_t *sft, int sft_num, int *sorted_id, FILE *out, FILE *fixed){

		char tmp_buffer[1024];
		int i,j;
		int score, score1, score2, thresh_score_s,thresh_score_p;
		char *tok, *t;

		thresh_score_s=309;//2 mismatch or 1 gap
		thresh_score_p=284;

		printf("fix start.\n");
		printf("sft_num:%i\n",sft_num);
		printf("buffer_len:%i\n",sft[sorted_id[0]].buffer_len);
		if(sft[sorted_id[0]].buffer_len%2){
			fprintf(stderr,"%s%i","odd buffer number.\n",sft[sorted_id[0]].buffer_len);
			exit(0);
		}else if(sft_num==1 && sft[sorted_id[0]].buffer_len==2){//unique mapped
			//parse buffer0
			parse_sam_field(sft[sorted_id[0]].buffer[0], sft[sorted_id[0]].sam_field, &sft[sorted_id[0]].field_num);
			if((atoi(sft[sorted_id[0]].sam_field[1]) & 0x04)== 0 && (atoi(sft[sorted_id[0]].sam_field[1]) & 0x08)== 0){//pair mapped
				printf("pair u\n");
				tok=strtok(sft[sorted_id[0]].sam_field[11],":");
				while(tok!=NULL){
					t=tok;
					tok = strtok (NULL, ":");
				}
				score1=atoi(t);
				//printf("score1:%i\n",score);
				if(score1>thresh_score_p){//buffer0 is high score
					//parse buffer1
					parse_sam_field(sft[sorted_id[0]].buffer[1], sft[sorted_id[0]].sam_field, &sft[sorted_id[0]].field_num);
					tok=strtok(sft[sorted_id[0]].sam_field[11],":");
					while(tok!=NULL){
						t=tok;
						tok = strtok (NULL, ":");
					}
					score2=atoi(t);
					//printf("score2:%i\n",score);
					if(score2>thresh_score_p){//both high score
						printf("write\n");
						n_pair_u++;
						fprintf(out,"%s", sft[sorted_id[0]].buffer[0]);
						fprintf(out,"%s", sft[sorted_id[0]].buffer[1]);
					}else{
						if(score1>thresh_score_s){//buffer0 is high score, buffer1 is not
							//parse buffer0
							parse_sam_field(sft[sorted_id[0]].buffer[0], sft[sorted_id[0]].sam_field, &sft[sorted_id[0]].field_num);
							int k;
							for(k=0;k<sft[sorted_id[0]].field_num-1;k++){
								if(k==1){//flag

									if((atoi(sft[sorted_id[0]].sam_field[k]) & 0x10)==0){//+strand
										fprintf(out,"%s\t","73");//flag = 01100011
									}else{
										fprintf(out,"%s\t","89");//flag = 01100011
									}

									//fprintf(fixed,"%s\t","99");
								}else if(k==6){//mate genome

									fprintf(out,"%s\t","*");
									//fprintf(fixed,"%s\t","=");
								}else if(k==7){//mate pos

									fprintf(out,"%i\t",0);
									//fprintf(fixed,"%i\t",mp2[j].pos);
								}else if(k==8){// isize

									fprintf(out,"%i\t",0);// rlen =36
									//fprintf(fixed,"%i\t",(mp2[j].pos - mp1[i].pos +36));// rlen =36
								}else{

									fprintf(out,"%s\t",sft[sorted_id[0]].sam_field[k]);
									//fprintf(fixed,"%s\t",sft[mp1[i].sft_id].sam_field[k]);
								}
							}
							fprintf(out,"%s",sft[sorted_id[0]].sam_field[k]);
						}
					}
				}else {//buffer0 is low score
					//parse buffer1
					parse_sam_field(sft[sorted_id[0]].buffer[1], sft[sorted_id[0]].sam_field, &sft[sorted_id[0]].field_num);
					strcpy(tmp_buffer,sft[sorted_id[0]].sam_field[11]);
					tok=strtok(tmp_buffer ,":");
					while(tok!=NULL){
						t=tok;
						tok = strtok (NULL, ":");
					}
					score2=atoi(t);
					if(score2>thresh_score_s){//score2 is high, score1 is not
						int k;
						for(k=0;k<sft[sorted_id[1]].field_num-1;k++){
							if(k==1){//flag

								if((atoi(sft[sorted_id[0]].sam_field[k]) & 0x10)==0){//+strand
									fprintf(out,"%s\t","73");//flag = 01100011
								}else{
									fprintf(out,"%s\t","89");//flag = 01100011
								}

								//fprintf(fixed,"%s\t","99");
							}else if(k==6){//mate genome

								fprintf(out,"%s\t","*");
								//fprintf(fixed,"%s\t","=");
							}else if(k==7){//mate pos

								fprintf(out,"%i\t",0);
								//fprintf(fixed,"%i\t",mp2[j].pos);
							}else if(k==8){// isize

								fprintf(out,"%i\t",0);// rlen =36
								//fprintf(fixed,"%i\t",(mp2[j].pos - mp1[i].pos +36));// rlen =36
							}else{

								fprintf(out,"%s\t",sft[sorted_id[1]].sam_field[k]);
								//fprintf(fixed,"%s\t",sft[mp1[i].sft_id].sam_field[k]);
							}
						}
						fprintf(out,"%s",sft[sorted_id[1]].sam_field[k]);
					}
				}

			}else{//single mapped
				printf("single u\n");
				if((atoi(sft[sorted_id[0]].sam_field[1]) & 0x04)== 0){//buffer0 is mapped
					tok=strtok(sft[sorted_id[0]].sam_field[11],":");
					while(tok!=NULL){
						t=tok;
						tok = strtok (NULL, ":");
					}
					score=atoi(t);
					printf("score1:%i\n",score);
					if(score>thresh_score_s){
						n_single_u++;
						fprintf(out,"%s", sft[sorted_id[0]].buffer[0]);
					}else{

					}
				}else if((atoi(sft[sorted_id[0]].sam_field[1]) & 0x08)== 0){
					parse_sam_field(sft[sorted_id[0]].buffer[1], sft[sorted_id[0]].sam_field, &sft[sorted_id[0]].field_num);
					tok=strtok(sft[sorted_id[0]].sam_field[11],":");
					while(tok!=NULL){
						t=tok;
						tok = strtok (NULL, ":");
					}
					score=atoi(t);
					printf("score2:%i\n",score);
					if(score>thresh_score_s){
						n_single_u++;
						fprintf(out,"%s", sft[sorted_id[0]].buffer[1]);
					}
				}
			}


			n_all_u++;
		}else {// multiple mapped
			fprintf(stdout,"buffer0:%s", sft[sorted_id[0]].buffer[0]);
			fprintf(stdout,"buffer1:%s", sft[sorted_id[0]].buffer[1]);
			int max_match=12;
			struct mapping_pos_t mp1[max_match], mp2[max_match];
			int k1,k2;
			int flag;

			//check if there is paired match

			k1=0;
			for(i=0;i<sft_num;i++){
				for(j=0;j<sft[sorted_id[i]].buffer_len;j+=2){
					//parse buffer j
					parse_sam_field(sft[sorted_id[i]].buffer[j], sft[sorted_id[i]].sam_field, &sft[sorted_id[i]].field_num);
					if((atoi(sft[sorted_id[i]].sam_field[1]) & 0x04)== 0 &&
							(atoi(sft[sorted_id[i]].sam_field[1]) & 0x08)== 0 ){//paired mapped

						tok=strtok(sft[sorted_id[i]].sam_field[11],":");
						while(tok!=NULL){
							t=tok;
							tok = strtok (NULL, ":");
						}
						score1=atoi(t);
						printf("score1:%i\n",score);
						if(score1>thresh_score_p){
							//parse buffer j+1
							parse_sam_field(sft[sorted_id[i]].buffer[j+1], sft[sorted_id[i]].sam_field, &sft[sorted_id[i]].field_num);
							tok=strtok(sft[sorted_id[i]].sam_field[11],":");
							while(tok!=NULL){
								t=tok;
								tok = strtok (NULL, ":");
							}
							score2=atoi(t);
							printf("score2:%i\n",score);
							if(score2>thresh_score_p){
								mp1[k1].sft_id=i;
								mp1[k1].buffer_id=j;
								k1++;
								if(k1>max_match){
									break;
								}
							}
						}
						//fprintf(out,"%s", sft[sorted_id[i]].buffer[j]);


					}
				}
				if(k1>max_match){
					break;
				}

			}

			printf("k1:%i\n",k1);
			if(k1){//there is paired match, do not fix
				if(k1>max_match){
					//too many map,  drop

				}else{
					//output paired match
					for(i=0;i<k1;i++){
						fprintf(out,"%s", sft[sorted_id[mp1[i].sft_id]].buffer[mp1[i].buffer_id]);
						fprintf(out,"%s", sft[sorted_id[mp1[i].sft_id]].buffer[mp1[i].buffer_id+1]);
					}
				}
				if(k1==1){
					n_pair_u++;
				}else{
					n_pair_m+=k1;
				}
			}else{//no paired match, try fixing
				flag=0;
				k1=0;
				k2=0;
				for(i=0;i<sft_num;i++){
					for(j=0;j<sft[sorted_id[i]].buffer_len;j++){
						//parse buffer j
						parse_sam_field(sft[sorted_id[i]].buffer[j], sft[sorted_id[i]].sam_field, &sft[sorted_id[i]].field_num);

						if((atoi(sft[sorted_id[i]].sam_field[1]) & 0x04)== 0){//mapped
							if((atoi(sft[sorted_id[i]].sam_field[1]) & 0x40)== 0x40){//1st in the pair

								tok=strtok(sft[sorted_id[i]].sam_field[11],":");
								while(tok!=NULL){
									t=tok;
									tok = strtok (NULL, ":");
								}

								if(atoi(t)>thresh_score_p){
									strcpy(mp1[k1].chr_name,sft[sorted_id[i]].sam_field[2]);
									mp1[k1].pos=atoi(sft[sorted_id[i]].sam_field[3]);
									mp1[k1].ascore=atoi(t);
									mp1[k1].flag=atoi(sft[sorted_id[i]].sam_field[1]);
									mp1[k1].sft_id=sorted_id[i];
									mp1[k1].buffer_id=j;

									k1++;
									if(k1==max_match){
										flag=1;
										break;
									}
								}


							}else{//2st
								tok=strtok(sft[sorted_id[i]].sam_field[11],":");
								while(tok!=NULL){
									t=tok;
									tok = strtok (NULL, ":");
								}
								if(atoi(t)>thresh_score_p){
									strcpy(mp2[k2].chr_name,sft[sorted_id[i]].sam_field[2]);
									mp2[k2].pos=atoi(sft[sorted_id[i]].sam_field[3]);
									mp2[k2].ascore=atoi(t);
									mp2[k2].sft_id=sorted_id[i];
									mp2[k2].buffer_id=j;
									mp2[k2].flag=atoi(sft[sorted_id[i]].sam_field[1]);
									k2++;
									if(k2==max_match){
										flag=1;
										break;
									}
								}


							}
						}else{
							//do nth.
						}
					}
					if(flag){//exceed max_match
						break;
					}
				}
				printf("flag:%i\n",flag);
				if(flag){//too many mapping pos. drop
					n_over_mapped++;
				}else{
					flag=0;
					printf("k1:%i\n",k1);
					printf("k2:%i\n",k1);
					char fix_buffer[max_match*1024];
					char *p=fix_buffer;
					memset(fix_buffer,0,sizeof(fix_buffer));
					for(i=0;i<k1;i++){
						for(j=0;j<k2;j++){

							if(strcmp(mp1[i].chr_name,mp2[j].chr_name)){//mapped to different chr
								//mapped to different chr, not included at the moment
								printf("mapped to different chr\n");
							}else{//mapped to same chr

								//invalid mapping distance, try to fix
								printf("try fixing.\n");
								//printf("mp1 flag:%i\n",mp1[i].flag);
								//printf("mp2 flag:%i\n",mp2[j].flag);
								//printf("mp1 pos:%i\n",mp1[i].pos);
								//printf("mp2 pos:%i\n",mp2[j].pos);

								//the scores are checked when mp1 & mp2 are filled
								if(((mp1[i].flag & 0x10) ==0) &&  ((mp2[j].flag & 0x10) ==0x10) && (mp1[i].pos < mp2[j].pos)){
									//mate1 mapped to +strand, mate2 mapped to -strand, mate1 is on the left of mate2
									flag++;
									int k=0;
									//parse mate 1
									parse_sam_field(sft[mp1[i].sft_id].buffer[mp1[i].buffer_id], sft[mp1[i].sft_id].sam_field, &sft[mp1[i].sft_id].field_num);
									//write mate1 to fix_buffer
									for(k=0;k<sft[mp1[i].sft_id].field_num-1;k++){
										if(k==1){//flag
											sprintf(p+strlen(fix_buffer),"%s\t","99");
											//fprintf(out,"%s\t","99");//flag = 01100011
											//fprintf(fixed,"%s\t","99");
										}else if(k==6){//mate genome
											sprintf(p+strlen(fix_buffer),"%s\t","=");
											//fprintf(out,"%s\t","=");
											//fprintf(fixed,"%s\t","=");
										}else if(k==7){//mate pos
											sprintf(p+strlen(fix_buffer),"%i\t",mp2[j].pos);
											//fprintf(out,"%i\t",mp2[j].pos);
											//fprintf(fixed,"%i\t",mp2[j].pos);
										}else if(k==8){// isize
											sprintf(p+strlen(fix_buffer),"%i\t",(mp2[j].pos - mp1[i].pos +36));
											//fprintf(out,"%i\t",(mp2[j].pos - mp1[i].pos +36));// rlen =36
											//fprintf(fixed,"%i\t",(mp2[j].pos - mp1[i].pos +36));// rlen =36
										}else{
											sprintf(p+strlen(fix_buffer),"%s\t",sft[mp1[i].sft_id].sam_field[k]);
											//fprintf(out,"%s\t",sft[mp1[i].sft_id].sam_field[k]);
											//fprintf(fixed,"%s\t",sft[mp1[i].sft_id].sam_field[k]);
										}
									}
									sprintf(p+strlen(fix_buffer),"%s",sft[mp1[i].sft_id].sam_field[k]);
									//fprintf(out,"%s",sft[mp1[i].sft_id].sam_field[k]);
									//fprintf(fixed,"%s",sft[mp1[i].sft_id].sam_field[k]);
									//parese mate 2
									parse_sam_field(sft[mp2[j].sft_id].buffer[mp2[j].buffer_id], sft[mp2[j].sft_id].sam_field, &sft[mp2[j].sft_id].field_num);
									//write mate2 to fix_buffer
									for(k=0;k<sft[mp2[j].sft_id].field_num-1;k++){
										if(k==1){//flag
											sprintf(p+strlen(fix_buffer),"%s\t","147");
											//fprintf(out,"%s\t","147");//flag = 10010011
											//fprintf(fixed,"%s\t","147");//flag = 10010011
										}else if(k==6){//mate genome
											sprintf(p+strlen(fix_buffer),"%s\t","=");
											//fprintf(out,"%s\t","=");
											//fprintf(fixed,"%s\t","=");
										}else if(k==7){//mate pos
											sprintf(p+strlen(fix_buffer),"%i\t",mp1[i].pos);
											//fprintf(out,"%i\t",mp1[i].pos);
											//fprintf(fixed,"%i\t",mp1[i].pos);
										}else if(k==8){// isize
											sprintf(p+strlen(fix_buffer),"%i\t",-1*(mp2[j].pos - mp1[i].pos +36));
											//fprintf(out,"%i\t",-1*(mp2[j].pos - mp1[i].pos +36));// rlen =36
											//fprintf(fixed,"%i\t",-1*(mp2[j].pos - mp1[i].pos +36));// rlen =36
										}else{
											sprintf(p+strlen(fix_buffer),"%s\t",sft[mp2[j].sft_id].sam_field[k]);
											//fprintf(out,"%s\t",sft[mp2[j].sft_id].sam_field[k]);
											//fprintf(fixed,"%s\t",sft[mp2[j].sft_id].sam_field[k]);
										}
									}
									sprintf(p+strlen(fix_buffer),"%s",sft[mp2[j].sft_id].sam_field[k]);
									//fprintf(out,"%s",sft[mp2[j].sft_id].sam_field[k]);
									//printf(fixed,"%s",sft[mp2[j].sft_id].sam_field[k]);

								}else if(((mp1[i].flag & 0x10) ==0x10) &&  ((mp2[j].flag & 0x10 ) ==0) && (mp1[i].pos > mp2[j].pos)){
									//mate1 mapped to -strand, mate2 mapped to +strand, mate1 is on the right of mate2
									flag++;
									int k=0;
									//pares mate1
									parse_sam_field(sft[mp1[i].sft_id].buffer[mp1[i].buffer_id], sft[mp1[i].sft_id].sam_field, &sft[mp1[i].sft_id].field_num);

									for(k=0;k<sft[mp1[i].sft_id].field_num-1;k++){
										if(k==1){//flag
											sprintf(p+strlen(fix_buffer),"%s\t","83");
											//fprintf(out,"%s\t","83");//flag = 01010011
											//fprintf(fixed,"%s\t","83");//flag = 01010011
										}else if(k==6){//mate genome
											sprintf(p+strlen(fix_buffer),"%s\t","=");
											//fprintf(out,"%s\t","=");
											//fprintf(fixed,"%s\t","=");
										}else if(k==7){//mate pos
											sprintf(p+strlen(fix_buffer),"%i\t",mp2[j].pos);
											//fprintf(out,"%i\t",mp2[j].pos);
											//fprintf(fixed,"%i\t",mp2[j].pos);
										}else if(k==8){// isize
											sprintf(p+strlen(fix_buffer),"%i\t",(mp1[j].pos - mp2[i].pos +36));// rlen =36
											//fprintf(out,"%i\t",(mp2[j].pos - mp1[i].pos +36));// rlen =36
											//fprintf(fixed,"%i\t",(mp2[j].pos - mp1[i].pos +36));// rlen =36
										}else{
											sprintf(p+strlen(fix_buffer),"%s\t",sft[mp1[i].sft_id].sam_field[k]);
											//fprintf(out,"%s\t",sft[mp1[i].sft_id].sam_field[k]);
											//fprintf(fixed,"%s\t",sft[mp1[i].sft_id].sam_field[k]);
										}
									}
									sprintf(p+strlen(fix_buffer),"%s",sft[mp1[i].sft_id].sam_field[k]);
									//fprintf(out,"%s",sft[mp1[i].sft_id].sam_field[k]);
									//fprintf(fixed,"%s",sft[mp1[i].sft_id].sam_field[k]);

									//parse mate2
									parse_sam_field(sft[mp2[j].sft_id].buffer[mp2[j].buffer_id], sft[mp2[j].sft_id].sam_field, &sft[mp2[j].sft_id].field_num);
									for(k=0;k<sft[mp2[j].sft_id].field_num-1;k++){
										if(k==1){//flag
											sprintf(p+strlen(fix_buffer),"%s\t","163");
											//fprintf(out,"%s\t","163");//flag = 10100011
											//fprintf(fixed,"%s\t","163");//flag = 10100011
										}else if(k==6){//mate genome
											sprintf(p+strlen(fix_buffer),"%s\t","=");
											//fprintf(out,"%s\t","=");
											//fprintf(fixed,"%s\t","=");
										}else if(k==7){//mate pos
											sprintf(p+strlen(fix_buffer),"%i\t",mp1[i].pos);
											//fprintf(out,"%i\t",mp1[i].pos);
											//fprintf(fixed,"%i\t",mp1[i].pos);
										}else if(k==8){// isize
											sprintf(p+strlen(fix_buffer),"%i\t",-1*(mp1[j].pos - mp2[i].pos +36));// rlen =36
											//fprintf(out,"%i\t",-1*(mp2[j].pos - mp1[i].pos +36));// rlen =36
											//fprintf(fixed,"%i\t",-1*(mp2[j].pos - mp1[i].pos +36));// rlen =36
										}else{
											sprintf(p+strlen(fix_buffer),"%s\t",sft[mp2[j].sft_id].sam_field[k]);
											//fprintf(out,"%s\t",sft[mp2[j].sft_id].sam_field[k]);
											//fprintf(fixed,"%s\t",sft[mp2[j].sft_id].sam_field[k]);
										}
									}
									sprintf(p+strlen(fix_buffer),"%s",sft[mp2[j].sft_id].sam_field[k]);
									//fprintf(out,"%s",sft[mp2[j].sft_id].sam_field[k]);
									//fprintf(fixed,"%s",sft[mp2[j].sft_id].sam_field[k]);
								}else{//invalid mate, do not fix
									printf("mp1 flag:%i\n",mp1[i].flag);
									printf("mp2 flag:%i\n",mp2[j].flag);
									printf("mp1 pos:%i\n",mp1[i].pos);
									printf("mp2 pos:%i\n",mp2[j].pos);
									printf("invalid mate.\n");
								}



							}

						}

					}
					printf("fix buffer:\n%s\n",fix_buffer);
					printf("flag:%i\n",flag);
					if(flag==0){//no mates are fixed, write some 1-mapped read here
						for(i=0;i<k1;i++){
							if(mp1[i].ascore>thresh_score_s){
								flag++;
							}
						}
						for(j=0;j<k2;j++){
							if(mp2[j].ascore>thresh_score_s){
								flag++;
							}
						}
						if(flag<max_match){
							//output k1 single read
							for(i=0;i<k1;i++){
								if(mp1[i].ascore>thresh_score_s){
									parse_sam_field(sft[mp1[i].sft_id].buffer[mp1[i].buffer_id], sft[mp1[i].sft_id].sam_field, &sft[mp1[i].sft_id].field_num);
									int k;
									for(k=0;k<sft[mp1[i].sft_id].field_num-1;k++){
										if(k==1){//flag
											if((mp1[i].flag & 0x10)==0){//+strand
												fprintf(out,"%s\t","73");//flag = 01100011
											}else{
												fprintf(out,"%s\t","89");//flag = 01100011
											}

											//fprintf(fixed,"%s\t","99");
										}else if(k==6){//mate genome

											fprintf(out,"%s\t","*");
											//fprintf(fixed,"%s\t","=");
										}else if(k==7){//mate pos

											fprintf(out,"%i\t",0);
											//fprintf(fixed,"%i\t",mp2[j].pos);
										}else if(k==8){// isize

											fprintf(out,"%i\t",0);// rlen =36
											//fprintf(fixed,"%i\t",(mp2[j].pos - mp1[i].pos +36));// rlen =36
										}else{

											fprintf(out,"%s\t",sft[mp1[i].sft_id].sam_field[k]);
											//fprintf(fixed,"%s\t",sft[mp1[i].sft_id].sam_field[k]);
										}
									}
									fprintf(out,"%s",sft[mp1[i].sft_id].sam_field[k]);
									//fprintf(out,"%s",sft[mp1[i].sft_id].buffer[mp1[i].buffer_id]);

								}
							}
							//output k2 single read
							for(j=0;j<k2;j++){
								if(mp2[j].ascore>thresh_score_s){
									parse_sam_field(sft[mp2[i].sft_id].buffer[mp2[i].buffer_id], sft[mp2[i].sft_id].sam_field, &sft[mp2[i].sft_id].field_num);
									int k;
									for(k=0;k<sft[mp2[i].sft_id].field_num-1;k++){
										if(k==1){//flag
											if((mp2[i].flag & 0x10)==0){//+strand
												fprintf(out,"%s\t","73");//flag = 01100011
											}else{
												fprintf(out,"%s\t","89");//flag = 01100011
											}

											//fprintf(fixed,"%s\t","99");
										}else if(k==6){//mate genome

											fprintf(out,"%s\t","*");
											//fprintf(fixed,"%s\t","=");
										}else if(k==7){//mate pos

											fprintf(out,"%i\t",0);
											//fprintf(fixed,"%i\t",mp2[j].pos);
										}else if(k==8){// isize

											fprintf(out,"%i\t",0);// rlen =36
											//fprintf(fixed,"%i\t",(mp2[j].pos - mp1[i].pos +36));// rlen =36
										}else{

											fprintf(out,"%s\t",sft[mp2[i].sft_id].sam_field[k]);
											//fprintf(fixed,"%s\t",sft[mp1[i].sft_id].sam_field[k]);
										}
									}
									fprintf(out,"%s",sft[mp2[i].sft_id].sam_field[k]);
									//fprintf(out,"%s",sft[mp2[j].sft_id].buffer[mp2[j].buffer_id]);
								}
							}
							n_low_score_m++;
						}
					}else if(flag<max_match){
						fprintf(out,"%s",fix_buffer);
						fprintf(fixed,"%s",fix_buffer);
						if(flag==1){
							n_fixed_u++;
						}else if(flag>1){
							n_fixed_m+=flag;
						}
					}else{

					}

				}
			}


		//free buffer1-4
		}

	//printf("fixed:%i\n",n_fixed);
	return 0;
}


int
sort_sam_field(struct sam_field_t *sft, int file_num, int field_id, int *sorted_id){
	int i,j,tmp;
	//char *sam_field[50];
	//int field_num;
	char tmp_buffer[1024];
	//char *name[file_num];

	//for(i=0;i<file_num;i++){
	//	name[i]=(char*)xcalloc(1024*sizeof(char));
	//}

	for(i=0;i<file_num;i++){
		strcpy(tmp_buffer, sft[i].buffer[0]);
		parse_sam_field(tmp_buffer,sft[i].sam_field, &sft[i].field_num);

	}
	for(i=0;i<file_num;i++){
		sorted_id[i]=i;
	}
	for(i=0;i<file_num;i++){
		for(j=i+1;j<file_num;j++){
			//printf("i is big:%i\n",sft[i].sam_field[field_id]>sft[j].sam_field[field_id]);
			if(strcmp(sft[i].sam_field[field_id],sft[j].sam_field[field_id])>0){
				tmp=sorted_id[i];
				sorted_id[i]=sorted_id[j];
				sorted_id[j]=tmp;
			}
		}
	}

	return 0;
}

int
merge_name_sorted_sam(int file_num,  char *out_file_name, char **file_name){
	int i, j;
	gzFile f[file_num];
	FILE *out, *fixed;
	char tmp_buffer[1024];
	int end_of_file[file_num], end_of_all;
	int sorted_id[file_num];
	char *name[file_num];
	char *last_buffer[file_num];
	int sft_num;
	//int thresh_score_s=285;

	int field_id=0;//sort by name


	n_unique=0;
	n_fixed=0;
	n_over_mapped=0;
	n_low_score_u=0;
	n_low_score_m=0;
	n_pair_u=0;
	n_single_u=0;
	n_pair_m=0;
	n_single_m=0;
	n_fixed_u=0;
	n_fixed_m=0;
	n_all_u=0;

	printf("merge start.\n");
	for(i=0;i<file_num;i++){
		printf("file %i:%s\n",i,file_name[i]);
	}
	printf("out put file:%s\n",out_file_name);
	for(i=0;i<file_num;i++){
		f[i]=gzopen(file_name[i],"r");
		if(f[i]==NULL){
			fprintf(stderr, "merge file open failed.%s\n",file_name[i]);
			exit(0);
		}
	}
	out=fopen(out_file_name,"w");
	if(out==NULL){
		fprintf(stderr, "out put file open failed.%s\n",out_file_name);
		exit(0);
	}

	char fixed_file[1024];
	sprintf(fixed_file,"%s_fixed",file_name[0]);
	fixed=fopen(fixed_file,"w");
	if(out==NULL){
		fprintf(stderr, "out put file open failed.%s\n","fixed.sam");
		exit(0);
	}
	struct sam_field_t *sft;
	sft=(sam_field_t*)xcalloc(file_num*sizeof(sam_field_t));
	for(i=0;i<file_num;i++){
		 sft[i].buffer_num=10;
		 sft[i].buffer=(char**)xcalloc(sft[i].buffer_num*sizeof(char*));
		 for(j=0;j< sft[i].buffer_num;j++){
		 	 sft[i].buffer[j]=(char *)xcalloc(1*1024*sizeof(char));
		 }

	}

	for(i=0;i<file_num;i++){
		 sft[i].sam_field=(char**)xcalloc(50*sizeof(char*));
		 for(j=0;j<50;j++){
			 sft[i].sam_field[j]=(char *)xcalloc(1*1024*sizeof(char));
		 }

	}

	for(i=0;i<file_num;i++){
		name[i]=(char *)xcalloc(1*1024*sizeof(char));
		last_buffer[i]=(char *)xcalloc(1*1024*sizeof(char));
	}

	for(i=0;i<file_num;i++){
		end_of_file[i]=0;
	}
	end_of_all=0;
	//read first row of all file to get names
	for(i=0;i<file_num;i++){
		while(gzgets(f[i],tmp_buffer,1024)!=NULL && tmp_buffer[0]=='@'){

		}
		sft[i].file_id=i;
		//sft[i].buffer_len=0;
		//parse_sam_field(tmp_buffer, sft[i].sam_field, &sft[i].field_num);
		//strcpy(sft[i].buffer[sft[i].buffer_len++],tmp_buffer);
		strcpy(last_buffer[i],tmp_buffer);
	}

	sft_num=file_num;
	for(i=0;i<file_num;i++){
		sorted_id[i]=i;
	}



	while(1){

		for(i=0;i<sft_num;i++){
			printf("%i\t%i\n",sft_num,sorted_id[i]);
		}

		//read 1 group of reads from each file
		for(i=0;i<sft_num;i++){
			if(end_of_file[sorted_id[i]]){
				continue;
			}
			//parse_sam_field(tmp_buffer, sft[i].sam_field, &sft[i].field_num);
			//if(field_id>field_num)
				//exit(0);
			sft[sorted_id[i]].buffer_len=0;
			strcpy(sft[sorted_id[i]].buffer[sft[sorted_id[i]].buffer_len++],last_buffer[sorted_id[i]]);
			parse_sam_field(last_buffer[sorted_id[i]], sft[sorted_id[i]].sam_field, &sft[sorted_id[i]].field_num);


			//exit(0);

			strcpy(name[sorted_id[i]], sft[sorted_id[i]].sam_field[field_id]);
			while(gzgets(f[sorted_id[i]],tmp_buffer,1024)!=NULL){
				if(tmp_buffer[0]=='@' || tmp_buffer[0]=='\n'){
					continue;
				}
				//printf("tmp buf:%s\n",tmp_buffer);
				parse_sam_field(tmp_buffer, sft[sorted_id[i]].sam_field, &sft[sorted_id[i]].field_num);

				if(strcmp(name[sorted_id[i]], sft[sorted_id[i]].sam_field[field_id])){//name changes
					strcpy(last_buffer[sorted_id[i]], tmp_buffer);
					//strcpy(name[i], sam_field[field_id]);
					break;
				}else{
					strcpy(sft[sorted_id[i]].buffer[sft[sorted_id[i]].buffer_len++],tmp_buffer);
					if(sft[sorted_id[i]].buffer_len==sft[sorted_id[i]].buffer_num){
						int p;
						sft[sorted_id[i]].buffer_num*=2;

						sft[sorted_id[i]].buffer=(char**)xrealloc(sft[sorted_id[i]].buffer,sft[sorted_id[i]].buffer_num*sizeof(char*));
						for(p=sft[sorted_id[i]].buffer_len;p<sft[sorted_id[i]].buffer_num;p++){
							sft[sorted_id[i]].buffer[p]=(char*)xcalloc(1*1024*sizeof(char));
						}
					}

				}
			}
			//if the name not changed , it means it gets to the end of file[sorted_id[i]]
			printf("name:%s\t%s a\n",name[sorted_id[i]],sft[sorted_id[i]].sam_field[field_id]);
			if(strcmp(name[sorted_id[i]], sft[sorted_id[i]].sam_field[field_id])){
				strcpy(name[sorted_id[i]], sft[sorted_id[i]].sam_field[field_id]);
			}else{
				sft[sorted_id[i]].buffer[0][1]='{';
				end_of_file[sorted_id[i]]=1;
			}

		}


		//sort the groups by name
		sort_sam_field(sft, file_num, field_id, sorted_id);
		for(i=0;i<file_num;i++){
			printf("sort id%i:%i\n",i,sorted_id[i]);
		}
		i=1;
		j=0;
		while(j<file_num-1 && !strcmp(sft[sorted_id[j]].sam_field[field_id],sft[sorted_id[j+1]].sam_field[field_id])){
			j++;
			i++;
		}
		sft_num=i;

		fix_and_filter(sft, i, sorted_id, out, fixed);

		for(i=0;i<file_num;i++){
			if(end_of_file[i]==0){
				end_of_all=0;
				break;
			}else{
				end_of_all=1;
			}

		}
		if(end_of_all){
			printf("merge end of all.\n");
			i=1;
			j=sft_num;
			printf("sft num:%i\n",sft_num);
			while(j<file_num){

				while(j<file_num-1 && !strcmp(sft[sorted_id[j]].sam_field[field_id],sft[sorted_id[j+1]].sam_field[field_id])){
					i++;
					j++;
				}
				//fix, filter by score and output
		//****************

				fix_and_filter(sft, i, sorted_id, out, fixed);

				j+=i;
			}
			break;
		}else{


		}


	}

	for(i=0;i<file_num;i++){
		gzclose(f[i]);
	}
	fclose(out);
	fclose(fixed);

	printf("n_unique:%i\n",n_unique);
	printf("n_fixed:%i\n",n_fixed);
	printf("n_over_mapped:%i\n",n_over_mapped);
	printf("n_low_score_u:%i\n",n_low_score_u);
	printf("n_low_score_m:%i\n",		n_low_score_m);
	printf("n_pair_u:%i\n",n_pair_u);
	printf("n_single_u:%i\n",n_single_u);
	printf("n_pair_m:%i\n",n_pair_m);
	printf("n_fixed_u:%i\n",n_fixed_u);
	printf("n_fixed_m:%i\n",n_fixed_m);

	printf("merge end.\n");
	return 0;
}
int
separate_1_2(char *sam_file){
	gzFile gf;
	char *sam_field[50];
	int field_num;
	FILE *f1,*f2;

	char out_file_1[1024];//for 1-mapped read
	char out_file_2[1024];//for 2-mapped read
	sprintf(out_file_1,"%s_1",sam_file);
	sprintf(out_file_2,"%s_2",sam_file);
	for(int i=0;i<50;i++){
		sam_field[i]=(char*)xcalloc(1024*sizeof(char));
	}

	gf=gzopen(sam_file,"r");
	if(gf==NULL){
		fprintf(stderr, "sam file open failed.%s\n",sam_file);
		exit(0);
	}
	f1=fopen(out_file_1,"w");
	f2=fopen(out_file_2,"w");
	if(f1==NULL || f2 ==NULL){
		fprintf(stderr, "out file open failed.\n");
		exit(0);
	}

	fprintf(f1, "%s\n","@HD	VN:1	SO:unsorted");
	fprintf(f1, "%s\n", "@SQ	SN:chr22	LN:49691432");
	fprintf(f2, "%s\n","@HD	VN:1	SO:unsorted");
	fprintf(f2, "%s\n", "@SQ	SN:chr22	LN:49691432");

	char *buffer[1000];
	int i;
	for(i=0;i<1000;i++){
		buffer[i]=(char*)xcalloc(1024*sizeof(char));
	}
	int load=0;
	bool more=true;
	while(more){
		for(load=0;load<1000;load++){
			if(gzgets(gf, buffer[load],1024)==NULL){
				more=false;
			}
		}
		for(i=0;i<load;i++){
			if(buffer[i][0]=='@'){
				continue;
			}
			parse_sam_field(buffer[i], sam_field, &field_num);

			if((atoi(sam_field[1]) & 0x04) == 0 && (atoi(sam_field[1]) & 0x08 )== 0){//mapped in pair
				//out put to file2
				fprintf(f2, "%s", buffer[i]);
			}else if((atoi(sam_field[1]) & 0x04)== 0){
				fprintf(f1, "%s", buffer[i]);
			}else{
				//drop unmapped read
			}
		}
	}


	fclose(f1);
	fclose(f2);
	gzclose(gf);
	printf("sep12 end.\n");
	return 0;
}
int
io_test(char *sam_file, bool use_buffer){
	gzFile gf;
	char tmp_buffer[1024];

	char *sam_field[50];
	int field_num;
	FILE *f1,*f2;

	char out_file_1[1024];//for 1-mapped read
	strcpy(out_file_1, "iotest.out") ;
	for(int i=0;i<50;i++){
		sam_field[i]=(char*)xcalloc(1024*sizeof(char));
	}

	gf=gzopen(sam_file,"r");
	if(gf==NULL){
		fprintf(stderr, "sam file open failed.%s\n",sam_file);
		exit(0);
	}
	f1=fopen(out_file_1,"w");

	if(f1==NULL){
		fprintf(stderr, "out file open failed.\n");
		exit(0);
	}



	if(!use_buffer){
		while(gzgets(gf,tmp_buffer,1024)!=NULL){
			if(tmp_buffer[0]=='@'){
				continue;
			}
			parse_sam_field(tmp_buffer, sam_field, &field_num);
			fprintf(f1,"%s\n",sam_field[0]);

		}
	}else{
		char *buffer[1000];
		int i;
		for(i=0;i<1000;i++){
			buffer[i]=(char*)xcalloc(1024*sizeof(char));
		}
		int load=0;
		bool more=true;
		while(more){
			for(load=0;load<1000;load++){
				if(gzgets(gf, buffer[load],1024)==NULL){
					more=false;
				}
			}
			for(i=0;i<load;i++){
				parse_sam_field(buffer[i], sam_field, &field_num);
				fprintf(f1,"%s\n",sam_field[0]);
			}
		}
	}

	fclose(f1);

	gzclose(gf);
	printf("sep12 end.\n");
	return 0;
}
int
trim_sam2(char *sam_file){
	gzFile gf;
	char *sam_field[50];
	int field_num;
	FILE *f1;
	int score1,score2;
	int thresh=284;
	char *tok,*t;
	char out_file_1[1024];//for 1-mapped read

	sprintf(out_file_1,"%s_trim",sam_file);

	for(int i=0;i<50;i++){
		sam_field[i]=(char*)xcalloc(1024*sizeof(char));
	}

	gf=gzopen(sam_file,"r");
	if(gf==NULL){
		fprintf(stderr, "sam file open failed.%s\n",sam_file);
		exit(0);
	}
	f1=fopen(out_file_1,"w");

	if(f1==NULL){
		fprintf(stderr, "out file open failed.\n");
		exit(0);
	}

	fprintf(f1, "%s\n","@HD	VN:1	SO:unsorted");
	fprintf(f1, "%s\n", "@SQ	SN:chr22	LN:49691432");

	char *buffer[1000];
	int i;
	for(i=0;i<1000;i++){
		buffer[i]=(char*)xcalloc(1024*sizeof(char));
	}
	int load=0;
	bool more=true;
	while(more){
		for(load=0;load<1000;load++){
			if(gzgets(gf, buffer[load],1024)==NULL){
				more=false;
			}
		}
		for(i=0;i<load;i+=2){
			if(buffer[i][0]=='@'){
				continue;
			}
			parse_sam_field(buffer[i], sam_field, &field_num);

			if((atoi(sam_field[1]) & 0x04) == 0 && (atoi(sam_field[1]) & 0x08 )== 0){//mapped in pair
				//out put to file2
				tok=strtok(sam_field[11],":");
				while(tok!=NULL){
					t=tok;
					tok = strtok (NULL, ":");
				}
				score1=atoi(t);
				if(score1>thresh){
					parse_sam_field(buffer[i+1], sam_field, &field_num);
					tok=strtok(sam_field[11],":");
					while(tok!=NULL){
						t=tok;
						tok = strtok (NULL, ":");
					}
					score2=atoi(t);
					if(score2>thresh){
						fprintf(f1, "%s", buffer[i]);
						fprintf(f1, "%s", buffer[i+1]);
					}
				}
			}else if((atoi(sam_field[1]) & 0x04)== 0){
				tok=strtok(sam_field[11],":");
				while(tok!=NULL){
					t=tok;
					tok = strtok (NULL, ":");
				}
				score1=atoi(t);
				if(score1>thresh){
					fprintf(f1, "%s", buffer[i]);
				}
			}else{
				parse_sam_field(buffer[i+1], sam_field, &field_num);
				tok=strtok(sam_field[11],":");
				while(tok!=NULL){
					t=tok;
					tok = strtok (NULL, ":");
				}
				score1=atoi(t);
				if(score1>thresh){
					fprintf(f1, "%s", buffer[i+1]);
				}
				//drop unmapped read
			}
		}
	}


	fclose(f1);

	gzclose(gf);
	printf("sep12 end.\n");
	return 0;
}
int
trim_sam(char *sam_file){
	gzFile gf;
	char *sam_field[50];
	int field_num;
	FILE *f1;
	int score1,score2;
	int thresh=284;
	char *tok,*t;
	char out_file_1[1024];//for 1-mapped read

	sprintf(out_file_1,"%s_trim",sam_file);

	for(int i=0;i<50;i++){
		sam_field[i]=(char*)xcalloc(1024*sizeof(char));
	}

	gf=gzopen(sam_file,"r");
	if(gf==NULL){
		fprintf(stderr, "sam file open failed.%s\n",sam_file);
		exit(0);
	}
	f1=fopen(out_file_1,"w");

	if(f1==NULL){
		fprintf(stderr, "out file open failed.\n");
		exit(0);
	}

	fprintf(f1, "%s\n","@HD	VN:1	SO:unsorted");
	fprintf(f1, "%s\n", "@SQ	SN:chr22	LN:49691432");

	char *buffer[1000];
	int i;
	for(i=0;i<1000;i++){
		buffer[i]=(char*)xcalloc(1024*sizeof(char));
	}
	int load=0;
	bool more=true;
	while(more){
		for(load=0;load<1000;load++){
			if(gzgets(gf, buffer[load],1024)==NULL){
				more=false;
			}
		}
		for(i=0;i<load;i++){
			if(buffer[i][0]=='@'){
				continue;
			}
			parse_sam_field(buffer[i], sam_field, &field_num);

			if((atoi(sam_field[1]) & 0x04) == 0 && (atoi(sam_field[1]) & 0x08 )== 0){//mapped in pair

				fprintf(f1, "%s", buffer[i]);

			}else if((atoi(sam_field[1]) & 0x04)== 0){
				tok=strtok(sam_field[11],":");
				while(tok!=NULL){
					t=tok;
					tok = strtok (NULL, ":");
				}
				score1=atoi(t);
				if(score1>thresh){
					fprintf(f1, "%s", buffer[i]);
				}
			}else{

				//drop unmapped read
			}
		}
	}


	fclose(f1);

	gzclose(gf);
	printf("sep12 end.\n");
	return 0;
}
int parse_fq(gzFile gf_fq, char **fq_line){
	int i;
	char buffer[1024];
	for(i=0;i<4;i++){
			if(gzgets(gf_fq, buffer,1024)==NULL){
					if(i!=0){
							fprintf(stderr, "fq file line num is not 4X.%i\n",i);
							exit(0);
					}else{
							return 0;
					}
			}else{
					strcpy(fq_line[i],buffer);
			}

	}
	return 1;
}
int
add_r2(char *sam_file, char *fq_file, char *chr){
        gzFile gf_sam, gf_fq;
        gf_sam=gzopen(sam_file,"r");
        FILE *fout;
        char buffer[1024];
        char *fq_line[4],*sam_field[50];
        int i;
        int field_num;
        char *p=NULL, *q=NULL;
        char out_file_1[1024];//for 1-mapped read
        sprintf(out_file_1,"%s_r2",sam_file);

        for(i=0;i<4;i++){
                fq_line[i]=(char*)xcalloc(1024*sizeof(char));
        }
        for(int i=0;i<50;i++){
                sam_field[i]=(char*)xcalloc(1024*sizeof(char));
        }
        if(gf_sam==NULL){
                fprintf(stderr, "sam file open failed.%s\n",sam_file);
                exit(0);
        }
        gf_fq=gzopen(fq_file,"r");
        if(gf_fq==NULL){
                fprintf(stderr, "fq file open failed.%s\n",fq_file);
                exit(0);
        }

        fout=fopen(out_file_1,"w");

        if(fout==NULL){
                fprintf(stderr, "out file open failed.\n");
                exit(0);
        }

        i=0;
        p=fq_line[0];
        while(gzgets(gf_sam, buffer,1024)!=NULL){
                if(buffer[0]=='@'){
                        continue;
                }
                parse_sam_field(buffer, sam_field, &field_num);
                if((atoi(sam_field[1]) & 0x04)==0x04){
                	//goto nextloop;
					continue;
				}
                if(strcmp(sam_field[2],chr)){//different chr or not mapped(chr is *), do not deal

                }else{
                        //printf("1:%s\n",sam_field[0]);
                        //sam_field[0][strlen(sam_field[0])-1]='\0';
                        q=strchr(sam_field[0],'/');
                        if(q!=NULL){
                        	printf("q:%s\n",q);
                        	*q='\0';
                        }
                        //strtok(sam_field[0],"pos");  //this is only for simulated data

                        if(strcmp(sam_field[0],p+1)!=0){
							   while(parse_fq(gf_fq,fq_line)){
									   //fq_line[0][strlen(fq_line[0])-2]='\0';
								   	   q=strchr(fq_line[0],'/');
								   	   if(q!=NULL){
								   		   *q = '\0';
								   	   }else{
								   		fq_line[0][strlen(fq_line[0])-1]='\0';
								   	   }
									   //strtok(fq_line[0],"pos");//this is only for simulated data
									   if(strcmp(sam_field[0],p+1)==0){//same read name
											   break;
									   }

							   }
					   }
                       //printf("1-1:%s\n",sam_field[0]);
                       //printf("2:%s\n",p+1);
                       //printf("3:%i\n",strcmp(sam_field[0],p+1));
					   assert(strcmp(sam_field[0],p+1)==0);
					   if(strcmp(sam_field[0],p+1)==0){

							  // printf("4:%s\n",p+1);
							  // printf("5:%i\n",strcmp(sam_field[0],p+1));
							   //sam_field[field_num-1][strlen(sam_field[field_num-1]-1)]='\0';
							   buffer[strlen(buffer)-1]='\t';

							   fprintf(fout, "%s", buffer);
							   fprintf(fout, "R2:Z:%s", fq_line[1]);
							   i++;
					   }else{
							   fprintf(stderr,"fq file is not enough.\n");
							   if(strlen(p)>0){
							   	  exit(0);
							   }
					   }
			   }

	   }
       printf("addr2 num:%i\n",i);

	   gzclose(gf_sam);
	   gzclose(gf_fq);
	   fclose(fout);
	   return 0;
}
int
aln_single_4_pindel(char *sam_file, char *fq_file, char *chr){
        gzFile gf_sam, gf_fq;
        gf_sam=gzopen(sam_file,"r");
        FILE *fout;
        char buffer[1024];
        char *fq_line[4],*sam_field[50];
        int i;
        int field_num;
        char *p=NULL, *q=NULL;
        char out_file_1[1024];//for 1-mapped read
        sprintf(out_file_1,"%s_r2",sam_file);

        for(i=0;i<4;i++){
                fq_line[i]=(char*)xcalloc(1024*sizeof(char));
        }
        for(int i=0;i<50;i++){
                sam_field[i]=(char*)xcalloc(1024*sizeof(char));
        }
        if(gf_sam==NULL){
                fprintf(stderr, "sam file open failed.%s\n",sam_file);
                exit(0);
        }
        gf_fq=gzopen(fq_file,"r");
        if(gf_fq==NULL){
                fprintf(stderr, "fq file open failed.%s\n",fq_file);
                exit(0);
        }

        fout=fopen(out_file_1,"w");

        if(fout==NULL){
                fprintf(stderr, "out file open failed.\n");
                exit(0);
        }

        i=0;
        p=fq_line[0];
        while(gzgets(gf_sam, buffer,1024)!=NULL){
                if(buffer[0]=='@'){
                        continue;
                }
                parse_sam_field(buffer, sam_field, &field_num);
                if((atoi(sam_field[1]) & 0x04)==0x04){
                	//goto nextloop;
					continue;
				}
                if(strcmp(sam_field[2],chr)){//different chr or not mapped(chr is *), do not deal

                }else{
                        //printf("1:%s\n",sam_field[0]);
                        //sam_field[0][strlen(sam_field[0])-1]='\0';
                        q=strchr(sam_field[0],'/');
                        if(q!=NULL){
                        	printf("q:%s\n",q);
                        	*q='\0';
                        }
                        //strtok(sam_field[0],"pos");  //this is only for simulated data

                        if(strcmp(sam_field[0],p+1)!=0){
							   while(parse_fq(gf_fq,fq_line)){
									   //fq_line[0][strlen(fq_line[0])-2]='\0';
								   	   q=strchr(fq_line[0],'/');
								   	   if(q!=NULL){
								   		   *q = '\0';
								   	   }else{
								   		fq_line[0][strlen(fq_line[0])-1]='\0';
								   	   }
									   //strtok(fq_line[0],"pos");//this is only for simulated data
									   if(strcmp(sam_field[0],p+1)==0){//same read name
											   break;
									   }

							   }
					   }
                       //printf("1-1:%s\n",sam_field[0]);
                       //printf("2:%s\n",p+1);
                       //printf("3:%i\n",strcmp(sam_field[0],p+1));
					   assert(strcmp(sam_field[0],p+1)==0);
					   if(strcmp(sam_field[0],p+1)==0){

							  // printf("4:%s\n",p+1);
							  // printf("5:%i\n",strcmp(sam_field[0],p+1));
							   //sam_field[field_num-1][strlen(sam_field[field_num-1]-1)]='\0';
							   //buffer[strlen(buffer)-1]='\t';
						   	   int j;
						   	   for(j=0;j<field_num;j++){
						   		   if(j==1){
						   			   fprintf(fout, "%i\t", 4);
						   		   }else if(j==2||j==5){
						   			   fprintf(fout, "%s\t", "*");
						   		   }else if(j==3 || j==4 || j==8){
						   			   fprintf(fout, "%s\t", "0");
						   		   }else if(j==6){
						   			   fprintf(fout, "%s\t", sam_field[2]);
						   		   }else if(j==7){
						   			   fprintf(fout, "%s\t", sam_field[3]);
						   		   }else if(j==9){
						   			   fq_line[1][strlen(fq_line[1])-1]='\t';
						   			   fprintf(fout, "%s", fq_line[1]);
						   		   }else if(j==field_num-1){
						   			   fprintf(fout, "%s", sam_field[j]);
						   		   }else{

						   		   	   fprintf(fout, "%s\t", sam_field[j]);
						   		   }
						   	   }

							   //fprintf(fout, "%s", buffer);
							   //fprintf(fout, "R2:Z:%s", fq_line[1]);
							   i++;
					   }else{
							   fprintf(stderr,"fq file is not enough.\n");
							   if(strlen(p)>0){
							   	  exit(0);
							   }
					   }
			   }

	   }
       printf("addr2 num:%i\n",i);

	   gzclose(gf_sam);
	   gzclose(gf_fq);
	   fclose(fout);
	   return 0;
}
int
get_sam_result(char *sam_file, char *chr){
        gzFile gf;
        char buffer[1024];
        char *sam_field[50];
        int field_num;

        for(int i=0;i<50;i++){
                sam_field[i]=(char*)xcalloc(1024*sizeof(char));
        }
        gf=gzopen(sam_file, "r");
        if(gf==NULL){
                fprintf(stderr,"sam file open failed.\n");
                exit(0);
        }
        FILE *f;
        char out_file_1[1024];//for 1-mapped read
        sprintf(out_file_1,"%s_%s",sam_file,chr);

        f=fopen(out_file_1,"w");
        if(f==NULL){
                fprintf(stderr,"out file open failed.\n");
                exit(0);
        }
        while(gzgets(gf,buffer,1024)){
                parse_sam_field(buffer, sam_field, &field_num);
                if(!strcmp(sam_field[2],chr)){
                        fprintf(f,"%s",buffer);
                }

        }
        gzclose(gf);
        fclose(f);
        return 0;

}
int
remove_single_read(char *shrimp_file, char *bowtie_file){//remove reads that mapped to other chr by bowtie in shrimp out
	gzFile gfs, gfb;
	char buffer[1024], buffer2[1024];
	char *sam_field[50];
	int field_num;
	int i;
	for(i=0;i<50;i++){
			sam_field[i]=(char*)xcalloc(1024*sizeof(char));
	}
	gfs=gzopen(shrimp_file, "r");
	gfb=gzopen(bowtie_file, "r");
	if(gfs==NULL || gfb==NULL){
			fprintf(stderr,"sam file open failed.\n");
			exit(0);
	}
	FILE *f;
	char out_file_1[1024];//for 1-mapped read
	sprintf(out_file_1,"%s_rm_bowtie",shrimp_file);

	f=fopen(out_file_1,"w");
	if(f==NULL){
			fprintf(stderr,"out file open failed.\n");
			exit(0);
	}
	while(gzgets(gfs,buffer,1024)){
		for(i=0;i<2;i++){
			if(gzgets(gfb,buffer2,1024)==NULL){
				fprintf(stderr,"shrimp file & bowtie file are not consisted.\n");
			}

		}
		parse_sam_field(buffer2, sam_field, &field_num);
		if((atoi(sam_field[1])&0x4)==0x0){
			fprintf(f,"%s",buffer);
		}

	}
	gzclose(gfs);
	gzclose(gfb);
	fclose(f);
	return 0;

}

void
parse_field(char *buffer, char **sam_field, int *field_num, char *delim){
	char *tok;
	int i;
	char tmp_buffer[1024];
	strcpy(tmp_buffer,buffer);
	tok=strtok(tmp_buffer, delim);
	i=0;
	while(tok!=NULL){
		strcpy(sam_field[i],tok);
		tok = strtok (NULL, delim);
		i++;
	}
	*field_num=i;


}
int
donor_pos_to_ref_pos(char *read_pos_file, char *chain_file){
	gzFile rpf, cf;
	char buffer[1024], buffer2[1024];
	char *sam_field[50],*sam_field2[50];
	int field_num;
	int i;


	for(i=0;i<50;i++){
			sam_field[i]=(char*)xcalloc(1024*sizeof(char));
			sam_field2[i]=(char*)xcalloc(1024*sizeof(char));
	}
	rpf=gzopen(read_pos_file, "r");
	cf=gzopen(chain_file, "r");
	if(rpf==NULL || cf==NULL){
			fprintf(stderr,"sam file open failed.\n");
			exit(0);
	}
	FILE *f;
	char out_file_1[1024];//for 1-mapped read
	sprintf(out_file_1,"%s_ref_pos",read_pos_file);

	f=fopen(out_file_1,"w");
	if(f==NULL){
			fprintf(stderr,"out file open failed.\n");
			exit(0);
	}

	int diff=0;
	int read_donor_pos=0;
	char *delim=" ";
	int *don_pos, *ref_pos;
	int chain_size=5000,chain_num=0;
	don_pos = (int*)xcalloc(chain_size*sizeof(int));
	ref_pos=(int*)xcalloc(chain_size*sizeof(int));

	while(gzgets(cf,buffer2,1024)){
		parse_field(buffer2, sam_field2, &field_num, delim);
		don_pos[chain_num]=atoi(sam_field2[1]);
		ref_pos[chain_num]=atoi(sam_field2[0]);
		chain_num++;
		if(chain_num==chain_size){
			chain_size*=2;
			don_pos=(int*)xrealloc(don_pos,chain_size*sizeof(int));
			ref_pos=(int*)xrealloc(ref_pos,chain_size*sizeof(int));
		}
	}
	while(gzgets(rpf,buffer,1024)){
		parse_field(buffer, sam_field, &field_num, delim);
		read_donor_pos=atoi(strstr(sam_field[1],"=")+1);
		printf("donor pos:%i\n",read_donor_pos);
		for(i=0;i<chain_num;i++){
			if(read_donor_pos < don_pos[i]){//continue reading chain file

				diff=read_donor_pos-don_pos[i-1];
				printf("pos ref:%i\tpos donor:%i\n",ref_pos[i],don_pos[i]);
				fprintf(f,"%s\t",sam_field[0]);
				fprintf(f,"%i\t",ref_pos[i-1]+diff);
				fprintf(f,"%s\t",sam_field[1]);
				fprintf(f,"%s\t",sam_field[2]);
				fprintf(f,"%s\t",sam_field[3]);
				fprintf(f,"%s\n",sam_field[4]);
				break;
			}
		}
		if(i==chain_num){//convert coordinate
			diff=read_donor_pos-don_pos[i-1];
			printf("pos ref:%i\tpos donor:%i\n",ref_pos[i-1],don_pos[i-1]);
			fprintf(f,"%s\t",sam_field[0]);
			fprintf(f,"%i\t",ref_pos[i-1]+diff);
			fprintf(f,"%s\t",sam_field[1]);
			fprintf(f,"%s\t",sam_field[2]);
			fprintf(f,"%s\t",sam_field[3]);
			fprintf(f,"%s\n",sam_field[4]);

		}
	}
	return 0;
}


